"""Floating-action-button to be displayed in absolute mode on the page. It will display a menu when hovered."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
import ipyvuetify as v

from ipywidgets import HTML, widgets, Layout
from IPython.display import display

try:
    from . import settings
    from . import fontsettings
    from . import Button
    from . import label
    from . import tooltip
except:
    import settings
    import fontsettings
    import Button
    import label
    import tooltip


#########################################################################################################################################
# Class to manage a customizable FAB menu
#########################################################################################################################################
class fab():
    """
    Floating-action-button to be displayed in absolute mode on the page. It will display a menu when hovered.
        
    Parameters
    ----------
    left : int or string, optional
        Absolute left position of the button (example: '800px' or '90%' or 750)
    top : int or string, optional
        Absolute top position of the button (example: '100px' or '10%' or 120)
    items : list of strings, optional
        Strings to be displayed as text of the options (default is [])
    onclick : list of function, optional
        Python functions to call when the user clicks on one of the items. One function for each of the items (default is [])
    tooltipitems : list of strings, optional
        Tooltip text for the items (default is [])
    iconsmall : bool, optional
        Flag to display the icon in small dimension (default is True)
    menumode : bool, optional
        Flag to display the options as dropdown menu. If False, the items are displayed as horizontally displaced toggle buttons (default is True)
    width : int, optional
        Width in pixel of the toggle buttons when menumode is False (default is 200)
    height : int, optional
        Height in pixel of the toggle buttons when menumode is False (default is 36)
    textcolor : str, optional
        Color to be used for the text of the buttons when menumode is False (default is None)
    selected : bool, optional
        If True the buttons have the settings.color_first as background (default is False)
    disabled : bool, optional
        If True the buttons are disabled (default is False)
    dark : bool, optional
        Flag that controls the color of the text in foreground (if True, the text will be displayed in white, elsewhere in black)
    large : bool, optional
        Flag that displays the fab button larger
    small : bool, optional
        Flag that displays the fab button smaller
    xsmall : bool, optional
        Flag that displays the fab button extra smaller
    outlined : bool, optional
        Flag that displays the fab button outlined
    textweight : int, optional
        Weight of the text to be shown in the label (default is 500, Bold is any value greater or equal to 500)
    zindex : int, optional
        Z-index of the fab button (default is 9999)
    output : ipywidgets.Output, optional
        Output widget on which the widget has to be displayed

    Example
    -------
    Creation and display of two fab widgets for the selection among 3 options::
        
        from vois.vuetify import fab
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()
        display(output)

        def on1():
            with output:
                print('selected 1')

        def on2():
            with output:
                print('selected 2')

        def on3():
            with output:
                print('selected 3')

        b1 = fab.fab(left='70%', top='150px',
                     items=['Option 1', 'Option 2', 'Option 3'],
                     tooltipitems=['Tooltip for Option 1'],
                     onclick=[on1,on2,on3], menumode=True,
                     width=180, selected=True,
                     dark=True, zindex=100, output=output)

        b2 = fab.fab(left='70%', top='250px',
                     items=['Option 1', 'Option 2', 'Option 3'],
                     tooltipitems=['Tooltip for Option 1'],
                     onclick=[on1,on2,on3], menumode=False,
                     width=180, selected=True,
                     dark=True, zindex=100, output=output)
                     
        b1.seticon('mdi-arrow-left-bold')
        b2.seticon('mdi-arrow-left',1)
        
        
    .. figure:: figures/fab.png
       :scale: 100 %
       :alt: fab widget

       Fab widget for selecting alternative options using a menu.

    .. figure:: figures/fab2.png
       :scale: 100 %
       :alt: fab widget as toggle buttons

       Fab widget for selecting alternative options using toggle buttons.
    """
    
   
    # Initialization
    def __init__(self, left=100, top=100, items=[], onclick=[], icon='mdi-arrow-down-thick', tooltipitems=[], iconsmall=True, menumode=True, 
                 width=200, height=36, textcolor=None, selected=False, disabled=False,  
                 dark=settings.dark_mode, large=False, small=False, xsmall=False, outlined=False, textweight=500, zindex=9999,
                 output=None):
        
        # Positioning
        self.left   = left
        self.top    = top
        self.zindex = zindex
        
        # Callback functions to call
        self.onclick  = onclick
        self.menumode = menumode
        self.width    = width
        
        self.nav = None

        self.iconsmall = iconsmall
        self.iconcolor = settings.textcolor_notdark
        if dark: self.iconcolor = settings.textcolor_dark
        
        if self.menumode:
            # Strings for the items to add to the menu
            self.items = items

            # Items in the menu
            self.listitems = []
            i = 0
            for item in items:
                li = v.ListItem(children=[item])
                li.on_event( 'click', self.__on_item_clicked)

                if type(tooltipitems) is list and i < len(tooltipitems):
                    li = tooltip.tooltip(tooltipitems[i],li)
                    
                self.listitems.append(li)
                i += 1

            # Creation of the menu
            self.button = v.Btn(v_on='menuData.on', color=settings.color_first, fab=True, dark=settings.dark_mode, depressed=True,
                                x_small=True, disabled=False, width=40, height=40, rounded=True, 
                                children=[v.Icon(small=self.iconsmall, color=self.iconcolor, children=[str(icon)])])
            
            self.menu = v.Menu(offset_y=True, open_on_hover=True, dense=True, v_slots=[{
                                              'name': 'activator',
                                              'variable': 'menuData',
                                              'children': self.button,
                                            }],
                          children=[v.List(children=self.listitems)] )
        else:
            # Strings for the buttons
            self.items = items
            
            if not textcolor is None: color = textcolor
            else:
                if selected: color = settings.color_first
                else:        color = settings.color_second
            
            # Buttons
            self.listitems = []
            self.buttons   = []
            i = 0
            for item in items:
                content = [item]
                if type(icon) is list and i < len(icon):
                    content.append(v.Icon(small=self.iconsmall, color=self.iconcolor, children=[icon[i]]))

                b = v.Btn(color=color, dark=dark, icon=False, depressed=True, outlined=outlined, large=large, small=small, x_small=xsmall, 
                          disabled=disabled, width=width-38, height=height, fab=True,
                          children=content, style_='font-family: %s; font-size: 17; font-weight: %d; text-transform: none' % (fontsettings.font_name, textweight), rounded=True)
                
                b.on_event( 'click', self.__on_item_clicked)
                self.buttons.append(b)
                
                if type(tooltipitems) is list and i < len(tooltipitems):
                    bt = tooltip.tooltip(tooltipitems[i],b)
                else:
                    bt = b
                
                self.listitems.append(bt)
                i += 1
            
            
        # Display of the panel
        if not output is None:
            with output:
                self.show(True)
    
    # Manage click menu item
    def __on_item_clicked(self, widget=None, event=None, data=None):
        txt = widget.children[0]
        if txt in self.items:
            index = self.items.index(txt)
            if index >= 0 and index < len(self.onclick):
                self.onclick[index]()
    
    # Set the icon of one of the buttons
    def seticon(self, iconname, index=0):
        """Set a different icon for the fab button"""
        if self.menumode:
            self.button.children = [v.Icon(small=self.iconsmall, color=self.iconcolor, children=[str(iconname)])]
        else:
            if index >= 0 and index < len(self.buttons):
                self.buttons[index].children = [self.buttons[index].children[0], v.Icon(small=self.iconsmall, color=self.iconcolor, children=[str(iconname)])]

                
    # Set the tooltip of one of the buttons (only if tooltip is already present and not in menumode!)
    def settooltip(self, text, index=0):
        """Change the tooltip text of one of the buttons"""
        if not self.menumode:
            if index >= 0 and index < len(self.listitems):
                if isinstance(self.listitems[index].children[0], v.generated.Tooltip):
                    self.listitems[index].children[0].children = [text]
        

        
    # Display/Hide the fab button
    def show(self, flag=True):
        """Show/hide the fab button"""
        if flag:
            if self.menumode:            
                self.nav = v.NavigationDrawer(stateless=True, permanent=True, floating=True, fixed=True, left=True, color="transparent", 
                                              width="'100px'", height="'100px'",
                                              style_="left:%s; top:%s; z-index:%d;" % (self.left,self.top,self.zindex), class_="pa-0 ma-0", children=[self.menu])
            else:
                # Add an element between any element of a list !!!
                def intersperse(lst, item):
                    result = [item] * (len(lst) * 2 - 1)
                    result[0::2] = lst
                    return result
                
                #buttons = intersperse(self.listitems, label.label('').draw())
                buttons = self.listitems
                
                if len(self.listitems) > 1: w = len(self.listitems)*(self.width - 30)
                else:                       w = self.width
                r = v.Row(no_gutters=True, justify="space-between", children=buttons)
                self.nav = v.NavigationDrawer(stateless=True, permanent=True, floating=True, fixed=True, left=True, color="transparent", 
                                              width='%dpx'%w, height='100px',
                                              style_="left:%s; top:%s; z-index:%d;" % (self.left,self.top,self.zindex), class_="pa-0 ma-0", children=[r])
                
            display(self.nav)
        else:
            if not self.nav is None:
                self.nav.close()
                
                
    # Close/hide the fab button
    def close(self):
        """Close/hide the fab button"""
        if not self.nav is None:
            self.nav.close()
            
        



"""Single selection widget from a dropdown list."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
import ipyvuetify as v

try:
    from . import settings
    from . import fontsettings
except:
    import settings
    import fontsettings


#####################################################################################################################################################
# Class to display a single selection widget
#####################################################################################################################################################
class selectSingle():
    """
    Single selection widget from a dropdown list. Passing the parameter newvalues_enabled=True enables the user to insert new strings in the widget.
        
    Parameters
    ----------
    label : str
        Help text to display 
    values : list of strings
        Strings to be displayed in the dropdown list of the widget 
    selection : str, optional
        String that is initially selected (default is '')
    mapping : function, optional
        Python function to call to transform the visible strings into codes (for example names of countries to their iso2 codes)
    reverse_mapping : function, optional
        Python function to call to transform the codes into visible strings (for example iso2 codes of countries to their names)
    onchange : function, optional
        Python function to call when the user selects one of the values in the list. The function will receive no parameter (use value property to retrieve the current selection)
    width : int, optional
        Width in pixel of the widget (default is 300 pixels)
    clearable : bool, optional
        Flag that controls if the widget should show to the user the 'X' button to clear its content (default is True)
    marginy : int, optional
        Margin in y coordinates to position the widget from top (default is 1)
    newvalues_enabled : bool, optional
        Flag that enables the user to insert new values beside those listed in the dropdown (default is False)
    newvalues_type : str, optional
        Type of the new values that the user can add in case newvalues_enabled is True (default is 'text')
    colorbackground : bool, optional
        Flag that controls the filling of the widget background with color when a value of the list is selected (default is False)
    color : str, optional
        Foreground color of the widget (default is settings.color_first)
            
    Example
    -------
    Creation and display of a single select widget for the selection of a country::
        
        from vois.vuetify import selectSingle
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()
        display(output)

        def onchange():
            with output:
                print(sel.value)

        sel = selectSingle.selectSingle('Country:',
                                        ['Belgium', 'France', 'Italy', 'Germany'],
                                        selection='France',
                                        width=200,
                                        onchange=onchange)
        sel.draw()

    .. figure:: figures/selectSingle.png
       :scale: 100 %
       :alt: selectSingle widget

       Example of a selectSingle widget to select from a list of countries.
   """
    
    # Initialization
    def __init__(self, label, values, selection='', mapping=None, reverse_mapping=None, width=300, onchange=None, clearable=True, marginy=1,
                       newvalues_enabled=False, newvalues_type='text', colorbackground=False, color=settings.color_first):
        
        self.label           = label
        self.values          = values
        self.mapping         = mapping           # Function to convert names to codes
        self.reverse_mapping = reverse_mapping   # Function to convert codes to names
        self.width           = width
        self.onchange        = onchange

        self.select = None
        self.value = selection
        
        self.colorbackground = colorbackground
        self._color          = color
        
        color,backcolor = self.__getColors()
        if newvalues_enabled:
            self.select = v.Combobox(v_model=self._value, label=self.label, dense=True, solo=False, outlined=True, multiple=False, chips=False, clearable=clearable, 
                                     item_color=color, color=color, class_='pa-0 mx-0 my-%d mb-n4' % marginy,
                                     background_color=backcolor,
                                     items=self.values, style_='max-width: %dpx; font-family: %s; font-weight:400; text-transform: none' % (self.width, fontsettings.font_name),
                                     type_=newvalues_type, autofocus=False, disabled=False)
            #self.select.on_event('keydown', self.__internal_onchange)
            #self.select.on_event('change', self.__internal_onchange)
        else:
            self.select = v.Select(v_model=self._value, label=self.label, dense=True, solo=False, outlined=True, multiple=False, chips=False, clearable=clearable, 
                                   item_color=color, color=color, class_='pa-0 mx-0 my-%d mb-n4' % marginy,
                                   background_color=backcolor, # menu_props="{ auto: true, maxHeight: 600 }",  # menu_props seems not working!
                                   items=self.values, style_='max-width: %dpx; font-family: %s; font-weight:400; text-transform: none' % (self.width, fontsettings.font_name), disabled=False)
        
        self.select.on_event('input', self.__internal_onchange)
        
        
    def __getColors(self):
        color     = self._color
        backcolor = 'white'
        if self.colorbackground and not self._value is None and len(str(self._value)) > 0 and self._value in self.values:
            color     = settings.select_textcolor
            backcolor = self._color
        return color,backcolor
        
        
    def __internal_onchange(self, widget, event, data):
        self._value = self.select.v_model
        color,backcolor = self.__getColors()
        self.select.color = color
        self.select.background_color = backcolor
        if self.onchange: self.onchange()

    def draw(self):
        """Returns the ipyvuetify object to display (the internal v.Html widget that contains a v.Select or a v.Combobox as its unique child)"""
        return v.Html(tag='div',children=[self.select], style_='overflow: hidden;')

    
    # value property
    @property
    def value(self):
        """
        Get/Set the selected value.
        
        Returns
        --------
        v : str
            text of the item currently selected

        Example
        -------
        Programmatically select one value::
            
            sel.value = 'Italy'
            print(sel.value)
        
        """
        if not self.mapping is None:
            return self.mapping(self._value)
        return self._value
    
    @value.setter
    def value(self, v):
        if not self.reverse_mapping is None:
            newv = self.reverse_mapping(v)
        else:
            newv = v
        
        if newv in self.values:
            self._value = newv

            if not self.select is None:
                self.select.v_model = self._value
                if self.onchange: self.onchange()
        else:
            if newv is None or len(newv) == 0:
                self._value = ''
            else:
                raise ValueError('Trying to set a value that is not in the list of allowed values.')


    # disabled property
    @property
    def disabled(self):
        """
        Get/Set the disabled state.
        """
        return self.select.disabled
    
    @disabled.setter
    def disabled(self, flag):
        self.select.disabled = flag
        

    # color property
    @property
    def color(self):
        """
        Get/Set the widget color.
        
        Returns
        --------
        c : str
            widget color

        Example
        -------
        Programmatically change the widget color::
            
            s.color = '#00FF00'
            print(s.color)
        
        """
        return self._color

    @color.setter
    def color(self, c):
        if isinstance(c, str):
            self._color = c
            color,backcolor = self.__getColors()
            self.select.color     = color
            self.background_color = backcolor
        


"""Button displaying an icon."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
import ipyvuetify as v

try:
    from . import settings
    from . import tooltip
except:
    import settings
    import tooltip

#####################################################################################################################################################
# Icon Button control
#####################################################################################################################################################
class iconButton():
    """
    Button displaying an icon.
        
    Parameters
    ----------
    icon : str, optional
        Icon to display inside the button (default is 'mdi-alert-outline')
    tooltip : str, optional
        Tooltip text for the button (default is '')
    color : str, optional
        Color used for the widget (default is the color_first defined in the settings.py module)
    outlined : bool, optional
        If True applies a thin border to the button (default is False)
    rounded : bool, optional
        If True the shape of the button is rounded (default is True)
    large : bool, optional
        If True makes the button large (default is False)
    small : bool, optional
        If True makes the button small (default is False)
    x_large : bool, optional
        If True makes the button extra-large (default is False)
    x_small : bool, optional
        If True makes the button extra-small (default is False)
    margins : str, optional
        Marging apply to the button (default is 'pa-0 ma-0')
    onclick : function, optional
        Python function to call when the user clicks on the button. The function will receive no parameters
    argument : any, optional
        Argument to be passed to the onclick funtion (default is None)
    disabled : bool, optional
        If True the button will be disabled (default is False)

    Example
    -------
    Creation and display of an icon button which changes color and tooltip on click::
        
        from vois.vuetify import iconButton
        from IPython.display import display
        
        def onclick():
            if b.color == 'red':
                b.color = 'amber'
                b.tooltip = 'Click to make the icon red'
            else:
                b.color = 'red'
                b.tooltip = 'Click to make the icon yellow'

        b = iconButton.iconButton(onclick=onclick, tooltip='Initial tooltip', x_large=True)
        display(b.draw())
   """

    # Initialization
    def __init__(self, icon='mdi-alert-outline', tooltip='', color=settings.color_first,
                 outlined=False, rounded=True, width=None,
                 large=False, small=False, x_large=False, x_small=False, margins='pa-0 ma-0',
                 onclick=None, argument=None,
                 disabled=False):
        
        self.icon      = icon
        self._tooltip  = tooltip
        self._color    = color
        self.outlined  = outlined
        self.rounded   = rounded
        self.width     = width
        self.large     = large
        self.small     = small
        self.x_large   = x_large
        self.x_small   = x_small
        self.margins   = margins
        self.onclick   = onclick
        self.argument  = argument
        self._disabled = disabled
        
        self.h = v.Html(tag='div',children=[])
        
        self.__createButton()
        
        
    # Creation of the button
    def __createButton(self):
        
        flagicon = True
        if self.outlined: flagicon = False
        self.btn = v.Btn(icon=flagicon, class_=self.margins, outlined=self.outlined, rounded=self.rounded,
                         large=self.large, small=self.small, x_large=self.x_large, x_small=self.x_small,
                         dark=settings.dark_mode, color=self._color, disabled=self._disabled,
                         width=self.width, min_width=self.width, max_width=self.width,
                         children=[v.Icon(children=[self.icon],
                                          large=self.large, small=self.small,
                                          x_large=self.x_large, x_small=self.x_small)])
        if not self.width is None:
            self.btn.min_width = self.width
            self.btn.max_width = self.width
            self.btn.width     = self.width
            
        self.btn.on_event('click.stop', self.__internal_onclick)
        
        if len(self._tooltip) > 0:
            obj = tooltip.tooltip(self._tooltip,self.btn)
        else:
            obj = self.btn
            
        self.h.children = [obj]
        
        
    # Manage click event
    def __internal_onclick(self, widget=None, event=None, data=None):
        if self.onclick:
            if not self.argument is None:
                self.onclick(self.argument)
            else:
                self.onclick()
        
    @property
    def color(self):
        """
        Get/Set the color of the button.
        """
        return self._color

    @color.setter
    def color(self, col):
        self._color = col
        self.__createButton()
        
        
    @property
    def tooltip(self):
        """
        Get/Set the tooltip of the button.
        """
        return self._tooltip

    @tooltip.setter
    def tooltip(self, tooltip):
        self._tooltip = tooltip
        self.__createButton()


    @property
    def disabled(self):
        """
        Get/Set the disabled state of the button.
        """
        return self._disabled

    @disabled.setter
    def disabled(self, flag):
        self._disabled = flag
        self.btn.disabled = self._disabled

        
    # Returns the vuetify object to display
    def draw(self):
        return self.h


"""Template page with left panel"""
# Author(s): Davide.De-Marchi@ec.europa.eu, Edoardo.Ramalli@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from ipywidgets import widgets, HTML, Layout, CallbackDispatcher
import ipyleaflet
from ipyleaflet import SearchControl, ScaleControl, FullScreenControl, WidgetControl
import ipyvuetify as v

# Vois imports
from vois.vuetify import settings, toggle, page

# Local imports
import mapUtils
import dynamicButton

# Panels dimensioning
LEFT_WIDTH = 400      # Width  in pixels of the left bar

# Name of layers
LAYERNAME_BACKGROUND  = 'Background'
LAYERNAME_LABELS      = 'Labels'


#####################################################################################################################################################
# Template page with left panel
#####################################################################################################################################################
class template1panel(page.page):


    # Initialization
    def __init__(self, output, onclose=None, **kwargs):
        super().__init__('Demo', 'Geospatial page with left panel', output, onclose=onclose, copyrighttext='European Commission - Joint Research Centre', **kwargs)


    #################################################################################################################################################
    # Create the page and returns the card widget where the content of the page must be displayed
    #################################################################################################################################################
    def create(self):
        super().create()
        
        # Initialize some member variables
        self.leftWidth = LEFT_WIDTH
        
        # Cards for the panels
        st = 'border-radius: 0px; border-color: %s; border-width: 1px; overflow: hidden;'%settings.color_first
        self.main_width  = 'calc(100vw - %dpx)'%self.leftWidth
        self.main_height = self.height
        self.cardLeft   = v.Card(flat=True, style_=st + 'border-right-width: 0px;', outlined=True, width=self.leftWidth, min_width=self.leftWidth, max_width=self.leftWidth, height=self.height)
        self.cardMain   = v.Card(flat=True, style_=st, outlined=True, width=self.main_width, height=self.main_height)
        
        # DynamicButtons to open/close the left and bottom panels
        self.dynbLeft = dynamicButton.dynamicButton(x1='%dpx'%(self.leftWidth-45), y1='64px', x2='48px', y2='64px', onclick1=self.leftClose, onclick2=self.leftOpen)
        
        # Creation of the contents for the panels
        self.createMain()
        self.createLeft()
        
        # Compose the panels
        self.card.children = [ self.dynbLeft.draw(), widgets.HBox([self.cardLeft, self.cardMain]) ]
        
        return self.card

    
    @property
    def titlecolor(self):
        return self._titlecolor
        
    @titlecolor.setter
    def titlecolor(self, color):
        page.page.titlecolor.fset(self, color)   # call super() property setter

        # Set the color of the panel outline!
        st = 'border-radius: 0px; border-color: %s; border-width: 1px; overflow: hidden;'%color
        self.cardLeft.style_ = st + 'border-right-width: 0px;'
        self.cardMain.style_ = st
        
        # Set the color of the dynamicButton
        self.dynbLeft.color = color
        
        # Set the color of basemaps toggle
        for b in self.toggleBasemap.buttons:
            b.color_selected = color
            if b._selected:
                b.b.color = color
    
            
    @property
    def footercolor(self):
        return self._footercolor
        
    @footercolor.setter
    def footercolor(self, color):
        page.page.footercolor.fset(self, color)   # call super() property setter

        # Set the color of basemaps toggle
        for b in self.toggleBasemap.buttons:
            b.color_unselected = color
            if not b._selected:
                b.b.color = color
        
        
    @property
    def titleheight(self):
        return self._titleheight
        
    @titleheight.setter
    def titleheight(self, height):
        page.page.titleheight.fset(self, height)   # call super() property setter

        self.main_height     = self.height
        self.cardLeft.height = self.height
        self.cardMain.height = self.main_height
        
        self.map.layout.height = 'calc(%s - 1.5px)'%self.height
        
        d = self._titleheight - 54
        newy = '%dpx'%(64+d)
        self.dynbLeft.y1 = self.dynbLeft.y2 = newy
        self.dynbLeft.setpos()
        

    @property
    def footerheight(self):
        return self._footerheight
        
    @footerheight.setter
    def footerheight(self, height):
        page.page.footerheight.fset(self, height)   # call super() property setter

        self.main_height     = self.height
        self.cardLeft.height = self.height
        self.cardMain.height = self.main_height
        
        self.map.layout.height = 'calc(%s - 1.5px)'%self.height
        
        d = self._titleheight - 54
        newy = '%dpx'%(64+d)
        self.dynbLeft.y1 = self.dynbLeft.y2 = newy
        self.dynbLeft.setpos()
        
                
    # Create the content of the left panel
    def createLeft(self):
        pass
    
    # Create the content of the Main panel
    def createMain(self):
        # Initial center and zoom of the map
        self.center = [50, 12]
        self.zoom   = 5

        # Map widget
        main_width  = 'calc(100vw - %dpx)'%self.leftWidth
        main_height = 'calc(%s - 1.5px)'%self.height
        self.map = ipyleaflet.Map(max_zoom=21, center=self.center, zoom=self.zoom, scroll_wheel_zoom=True, 
                                  basemap=mapUtils.EmptyBasemap(), attribution_control=False, layout=Layout(width=main_width, height=main_height, margin='0px 0px 0px 0px'))
        
        mapUtils.addLayer(self.map, mapUtils.OSM_EC(),      LAYERNAME_BACKGROUND)
        mapUtils.addLayer(self.map, mapUtils.CartoLabels(), LAYERNAME_LABELS)
        layer = mapUtils.getLayer(self.map, LAYERNAME_LABELS)
        layer.opacity = 0.0
        
        
        # Add FullScreen control
        self.map.add_control(FullScreenControl(position="topright"))

        # Add Search control
        self.map.add_control(SearchControl(position="topleft",url='https://nominatim.openstreetmap.org/search?format=json&q={s}',zoom=12))

        # Add Scale control
        self.map.add_control(ScaleControl(position='topleft'))

        # Add widget control to select basemaps
        self.toggleBasemap = toggle.toggle(0,
                                           ['Gisco', 'Esri', 'Google'],
                                           tooltips=['Select EC Gisco roadmap as background layer', 'Select ESRI WorldImagery as background layer', 'Select GOOGLE Satellite as background layer'],
                                           colorselected=settings.color_first, colorunselected=settings.color_second, rounded=False,
                                           dark=settings.dark_mode, onchange=self.onSelectBasemap, row=True, width=70, justify='start', paddingrow=0, tile=True)
        wcBasemap = WidgetControl(widget=self.toggleBasemap.draw(), position='bottomright')
        self.map.add_control(wcBasemap)
        
        
        # Add widget control to display geographic coordinates at mouse move
        self.cardCoordinates = v.Card(flat=True)
        wcCoordinates = WidgetControl(widget=self.cardCoordinates, position='topright')
        self.map.add_control(wcCoordinates)
        
        # Setup interaction on the map
        self.map._interaction_callbacks = CallbackDispatcher()
        self.map.on_interaction(self.handleMapInteraction)
        
        # Display the map inside the main card
        self.cardMain.children = [self.map]
        

    # Manage all user interaction on the map
    def handleMapInteraction(self, **kwargs):
        if kwargs.get('type') == 'mousemove':
            lon = kwargs.get('coordinates')[1]
            lat = kwargs.get('coordinates')[0]
            self.cardCoordinates.children = [v.Html(tag='div', children=[' %.4f° N, %.4f° E'%(lat,lon)], style_='color: black;', class_='pa-0 ma-0 ml-1 mr-1')]
        #elif not self.handleInteractionStations(**kwargs):
        #    self.handleInteractionModel(**kwargs)
        
        

    # Selection of a basemap
    def onSelectBasemap(self, index):
        layer = mapUtils.getLayer(self.map, LAYERNAME_LABELS)

        if index == 1:
            basemaplayer = mapUtils.EsriWorldImagery()
            layer.opacity = 1.0
        elif index == 2:
            basemaplayer = mapUtils.GoogleHybrid()
            layer.opacity = 0.0
        else:
            basemaplayer = mapUtils.OSM_EC()
            layer.opacity = 0.0

        mapUtils.addLayer(self.map, basemaplayer, LAYERNAME_BACKGROUND)
    
    
    #################################################################################################################################################
    # Manage the opening/closing of the dynamic panels (left and bottom)
    #################################################################################################################################################
        
    # Close the left panel
    def leftClose(self):
        self.leftWidth = 0
        self.main_width  = '100vw'
        self.cardLeft.width     = self.leftWidth
        self.cardLeft.min_width = self.leftWidth
        self.cardLeft.max_width = self.leftWidth
        
        self.cardMain.width   = self.main_width
        self.map.layout.width = self.main_width
        

    # Open the left panel
    def leftOpen(self):
        self.leftWidth = LEFT_WIDTH
        self.main_width = 'calc(100vw - %dpx)'%self.leftWidth
        self.cardLeft.width     = self.leftWidth
        self.cardLeft.min_width = self.leftWidth
        self.cardLeft.max_width = self.leftWidth
        
        self.cardMain.width   = self.main_width
        self.map.layout.width = self.main_width



import warnings
from typing import Callable, Dict, Any
import functools
from functools import partial


def deprecation_class_warning(old_class_name: str):
    warnings.warn('The {} class is deprecated, please use {}.'.format(old_class_name,
                                                                                          old_class_name.capitalize()),
                  category=DeprecationWarning,
                  stacklevel=2)


def deprecated_init_alias(**aliases: str) -> Callable:
    """Decorator for deprecated function and method arguments.

    Use as follows:

    @deprecated_alias(old_arg='new_arg')
    def myfunc(new_arg):
        ...

    """

    def deco(f: Callable):
        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            rename_kwargs(f.__name__, kwargs, aliases, args[0])
            return f(*args, **kwargs)

        return wrapper

    return deco


def rename_kwargs(func_name: str, kwargs: Dict[str, Any], aliases: Dict[str, str], obj=None):
    """Helper function for deprecating function arguments."""
    for alias, new in aliases.items():
        if alias in kwargs:
            if new in kwargs:
                raise TypeError(
                    f"{func_name} received both '{alias}' and '{new}' as arguments!"
                    f" '{alias}' is deprecated, use '{new}' instead."
                )
            warnings.warn(
                message=(
                    f"'{alias}' is deprecated as an argument to '{func_name}'; use"
                    f" '{new}' instead."
                ),
                category=DeprecationWarning,
                stacklevel=2,
            )

            kwargs[new] = kwargs.pop(alias)


def create_deprecated_alias(c_obj, alias, new):
    deprecation_message = "Do not use it anymore! '{}' is deprecated as property; use '{}' instead".format(alias, new)

    def generic_old_getter(c_obj):
        warnings.warn(
            message=deprecation_message,
            category=DeprecationWarning,
            stacklevel=2,
        )
        return getattr(c_obj, new)

    def generic_old_setter(c_obj, c_value):
        warnings.warn(
            message=deprecation_message,
            category=DeprecationWarning,
            stacklevel=2,
        )
        setattr(c_obj, new, c_value)

    setattr(c_obj.__class__, alias, property(fget=generic_old_getter, fset=generic_old_setter))


def create_deprecated_alias_2(c_obj, aliases):
    for alias, new in aliases.items():
        print(id(c_obj), alias, new)

        # c_value = getattr(c_obj, new)

        # print(c_value, alias, new, id(c_obj))

        deprecation_message = "Do not use it anymore! '{}' is deprecated as property; use '{}' instead".format(alias,
                                                                                                               new)

        def generic_old_getter(c_obj):
            warnings.warn(
                message=deprecation_message,
                category=DeprecationWarning,
                stacklevel=2,
            )
            return getattr(c_obj, new)

        def generic_old_setter(c_obj, c_value):
            warnings.warn(
                message=deprecation_message,
                category=DeprecationWarning,
                stacklevel=2,
            )
            setattr(c_obj, new, c_value)

        setattr(c_obj.__class__, alias,
                property(fget=generic_old_getter, fset=partial(generic_old_setter, c_value=getattr(c_obj, new))))



"""Button widget to call a python function when clicked."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

import ipyvuetify as v

from vois.vuetify.utils.util import *
from typing import Callable, Any, Union, Optional


#######################################################################################################################
# Button class. On click python function called when clicked
# Uses settings font to display the button text and interactivity when hover
#######################################################################################################################
class Button(v.Html):
    """
    Button widget to call a python function when clicked.
        
    Parameters
    ----------
    text : str
        Test string to be displayed on the button widget
    on_click : function, optional
        Python function to call when the user clicks on the button. The function will receive as parameter the value of the argument (default is None)
    on_dblclick : function, optional
        Python function to call when the user double-clicks on the button. The function will receive as parameter the value of the argument (default is None)
    argument : any, optional
        Argument to be passed to the onclick function when user click on the label (default is None)
    width : int, optional
        Width of the button widget in pixels (default is 100)
    height : int, optional
        Height of the button widget in pixels (default is 36)
    selected : bool, optional
        Flag to show the button as selected (default is False)
    disabled : bool, optional
        Flag to show the button as disabled (default is False)
    tooltip : str, optional
        Tooltip text to show when the user hovers on the button (default is '')
    large : bool, optional
        Flag that sets the large version of the button (default is False)
    x_large : bool, optional
        Flag that sets the xlarge version of the button (default is False)
    small : bool, optional
        Flag that sets the small version of the button (default is False)
    x_small : bool, optional
        Flag that sets the xsmall version of the button (default is False)
    outlined : bool, optional
        Flag to show the button as outlined (default is False)
    text_weight : int, optional
        Weight of the text to be shown in the label (default is 500, Bold is any value greater or equal to 500)
    href : str, optional
        URL to open when the button is clicked (default is None)
    target : str, optional
        Designates the target attribute (where the URL page is opened, for instance: '_blank' to open it in a new browser tab). This should only be applied when using the href parameter (default is None)
    only_text : bool, optional
         If True, the button will contain only the text (default is False)
    text_color : str, optional
        Color used for the button text (default is None)
    icon: str, optional
        Name of the icon to display aside the text of the label (default is None)
    icon_large : bool, optional
        Flag that sets the large version of the icon (default is False)
    icon_small : bool, optional
        Flag that sets the small version of the icon (default is False)
    icon_left : bool, optional
        Flag that sets the position of the icon  to the left of the text of the label (default is False)
    icon_color : str, optional
        Color of the icon (default is 'black' if settings.dark_mode is False and 'white if settings.dark_mode is True)
    auto_select : bool, optional
        If True, the button becomes selected when clicked (default is False)
    dark : bool, optional
        Flag to invert the text and backcolor (default is the value of settings.dark_mode)
    rounded : bool, optional
        Flag to display the button with rounded corners (default is the value of settings.button_rounded)
    tile : bool, optional
        Flag to remove the button small border (default is False)
    color_selected : str, optional
        Color used for the button when it is selected (default is settings.color_first)
    color_unselected : str, optional
        Color used for the button when it is not selected (default is settings.color_second)
            
    Note
    ----
    All the icons from https://materialdesignicons.com/ site can be used, just by prepending 'mdi-' to their name.
    
    All the free icons from https://fontawesome.com/ site can be used, just by prepending 'fa-' to their name.
    
    
    Example
    -------
    Creation and display of a some button widgets playing with the parameters::
        
        from vois.vuetify import settings, Button

        def onclick(arg=None):
            if arg==1: b1.selected = not b1.selected
            if arg==2: b2.selected = not b2.selected
            else:      b3.selected = not b3.selected

        b1 = Button('Test button 1', text_weight=300, on_click=onclick, argument=1,
                    width=150, height=36, 
                    tooltip='Tooltip for button 1', selected=False, rounded=True,
                    icon='mdi-car-light-high', iconColor='black')

        b2 = Button('Test button 2', text_weight=450, on_click=onclick, argument=2,
                    width=150, height=48,
                    tooltip='Tooltip for button 2', selected=True, rounded=False)

        b3 = Button('Test button 3', text_weight=450, on_click=onclick, argument=3,
                    width=150, height=38,
                    text_color=settings.color_first,
                    tooltip='Tooltip for button 3', outlined=True, rounded=True)

        b4 = Button('Contacts', only_text=True, text_color=settings.color_first,
                    width=150, height=28,
                    href='https://ec.europa.eu/info/contact_en', target="_blank",
                    tooltip='Open a URL')

        display(b1)
        display(b2)
        display(b3)
        display(b4)


    .. figure:: figures/button.png
       :scale: 100 %
       :alt: button widget

       Example of a 4 button widgets with different display modes.
   """
    deprecation_alias = dict(onclick='on_click', xlarge='x_large', xsmall='x_small', textweight='text_weight',
                             onlytext='only_text', textcolor='text_color', iconlarge='icon_large',
                             iconsmall='icon_small',
                             iconleft='icon_left', iconcolor='icon_color', autoselect='auto_select',
                             colorselected='color_selected', colorunselected='color_unselected',
                             ondblclick='on_dblclick')

    # Initialization
    @deprecated_init_alias(**deprecation_alias)
    def __init__(self, text: str,
                 on_click: Optional[Union[Callable[[], None], Callable[[dict[str, Any]], None]]] = None,
                 argument: Optional[Any] = None,  # TODO forzare a dict per **kwargs
                 width: int = 100,
                 height: int = 36,
                 selected: bool = False,
                 disabled: bool = False,
                 tooltip: str = '',
                 large: bool = False,
                 x_large: bool = False,
                 small: bool = False,
                 x_small: bool = False,
                 outlined: bool = False,
                 text_weight: int = 500,
                 href: Optional[str] = None,
                 target: Optional[str] = None,
                 only_text: bool = False,
                 text_color: Optional[str] = None,
                 class_: str = "pa-0 ma-0",
                 icon: Optional[str] = None,
                 icon_large: bool = False,
                 icon_small: bool = False,
                 icon_left: bool = False,
                 icon_color: str = None,
                 auto_select: bool = False,
                 dark: bool = None,
                 rounded: bool = None,
                 tile: bool = False,
                 color_selected: str = None,
                 color_unselected: str = None,
                 on_dblclick: Optional[Union[Callable[[], None], Callable[[dict[str, Any]], None]]] = None,
                 style_: str = '',
                 **kwargs) -> None:

        from vois.vuetify import settings, fontsettings
        
        super().__init__(**kwargs)

        self.on_click = on_click
        self.on_dblclick = on_dblclick
        self.argument = argument
        self._selected = selected
        self._disabled = disabled
        self.auto_select = auto_select
        self._text = text
        self.icon_large = icon_large
        self.icon_small = icon_small
        self.color_selected   = settings.color_first  if color_selected is None else color_selected
        self.color_unselected = settings.color_second if color_unselected is None else color_unselected
        self._dark            = settings.dark_mode if dark is None else dark
        self._rounded         = settings.button_rounded if rounded is None else rounded
        if icon_color is None:
            self.icon_color = 'white' if settings.dark_mode else 'black'
        else:
            self.icon_color = icon_color
                
        self._icon = icon

        self.icon_distance = " ml-2"

        if text_color:
            color = text_color
        else:
            color = self.color_selected if self._selected else self.color_unselected

        if self._icon is None:
            childs = [self._text]
        else:
            if not self._text:
                self.icon_distance = ""
            elif icon_left:
                self.icon_distance = " mr-2"

            icn = v.Icon(class_="pa-0 ma-0 %s" % self.icon_distance, large=self.icon_large, small=self.icon_small,
                         color=self.icon_color, children=[self._icon])
            if icon_left:
                if not self._text:
                    childs = [icn]
                else:
                    childs = [icn, self._text]
            else:
                if not self._text:
                    childs = [icn]
                else:
                    childs = [self._text, icn]

        self.b = v.Btn(color=color, dark=self._dark, icon=only_text, depressed=True, outlined=outlined, large=large,
                       x_large=x_large, small=small, x_small=x_small,
                       disabled=disabled, width=width, min_width=width, height=height, min_height=height, href=href,
                       target=target, tile=tile,
                       children=childs,
                       style_='font-family: %s; font-size: 17; font-weight: %d; text-transform: none; ' % (
                           fontsettings.font_name, text_weight) + style_,
                       rounded=self._rounded)

        self.b.on_event('click', self.__internal_onclick)
        self.b.on_event('dblclick', self.__internal_ondblclick)

        if len(tooltip) > 0:
            self.b.v_on = 'tooltip.on'
        self.container = v.Container(class_=class_, children=[
            v.Tooltip(color=settings.tooltip_backcolor, transition="scale-transition", bottom=True,
                      v_slots=[{'name': 'activator', 'variable': 'tooltip', 'children': self.b}],
                      children=[tooltip])])

        self.tag = 'div'
        self.children = [self.container]

        for alias, new in self.deprecation_alias.items():
            create_deprecated_alias(self, alias, new)

    def draw(self):
        warnings.warn('The "draw" method is deprecated, please just use the object widget itself.',
                      category=DeprecationWarning,
                      stacklevel=2)
        return self

    # Manage click event
    def __internal_onclick(self, widget=None, event=None, data=None):
        if self.on_click:
            if self.argument is not None:
                self.on_click(self.argument)  # TODO switch to kwargs
            else:
                self.on_click()
        if self.auto_select:
            self.selected = True

    # Manage dblclick event
    def __internal_ondblclick(self, widget=None, event=None, data=None):
        if self.on_dblclick:
            if self.argument:  # TODO usare argument diversi da onclick
                self.on_dblclick(self.argument)
            else:
                self.on_dblclick()

    @property
    def selected(self):
        """
        Get/Set the selected state of the button widget.
        
        Returns
        --------
        selected status : bool
            True if the button is selected, False otherwise

        Example
        -------
        Programmatically select a button::
            
            b.selected = True
            print(b.selected)
        """
        return self._selected

    @selected.setter
    def selected(self, flag):
        self._selected = bool(flag)
        if self._selected:
            color = self.color_selected
        else:
            color = self.color_unselected
        self.b.color = color

    @property
    def disabled(self):
        """
        Get/Set the disabled state of the button widget.
        
        Returns
        --------
        disabled status : bool
            True if the button is disabled, False otherwise
        """
        return self._disabled

    @disabled.setter
    def disabled(self, flag):
        self._disabled = bool(flag)
        self.b.disabled = self._disabled

    # Change the icon for the button
    def setIcon(self, iconname):  # TODO da fare come text
        """
        Change the icon for the button

        Example
        -------
        Creation of a button and programmatically change its icon::
                
            from vois.vuetify import settings, button
            
            b = Button('Test button', text_weight=450, width=150, height=46,
                       selected=True, rounded=True,
                       icon='mdi-menu-open', icon_color='black', icon_large=True)
            display(b.draw())
            b.setIcon('mdi-menu')

        """
        warnings.warn("This method is deprecate used instead button_obj.icon = 'your_icon_name'", DeprecationWarning,
                      stacklevel=2)
        self.icon = iconname

    @property
    def icon(self):
        return self._icon

    @icon.setter
    def icon(self, icon_name: str):
        """
        Change the icon for the button

        Example
        -------
        Creation of a button and programmatically change its icon::

            from vois.vuetify import settings, Button

            b = Button('Test button', text_weight=450, width=150, height=46,
                       selected=True, rounded=True,
                       icon='mdi-menu-open', icon_color='black', icon_large=True)
            display(b)
            b.icon = 'mdi-menu'

        """
        self._icon = icon_name
        for item in self.b.children:
            if type(item).__name__ == 'Icon':
                newicon = v.Icon(class_="pa-0 ma-0 %s" % self.icon_distance, large=self.icon_large,
                                 small=self.icon_small,
                                 color=self.icon_color, children=[self._icon])
                self.b.children = [newicon if x == item else x for x in self.b.children]
                break

    # Change the text for the button
    def setText(self, newtext: str):
        """
        Change the text for the button

        Example
        -------
        Creation of a button and programmatically change its text::

            from vois.vuetify import settings, Button

            b = Button('Test button', text_weight=450, width=250, height=46,
                       selected=True, rounded=True)
            display(b.draw())
            b.setText('New button text')

        """
        warnings.warn("This method is deprecate used instead button_obj.text = 'your_new_text'", DeprecationWarning,
                      stacklevel=2)
        self.text = newtext

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, value):
        """
        Change the text for the button

        Example
        -------
        Creation of a button and programmatically change its text::

            from vois.vuetify import settings, Button

            b = Button('Test button', text_weight=450, width=250, height=46,
                       selected=True, rounded=True)
            display(b)
            b.text = 'New button text'

        """
        tmp = self.b.children
        tmp[tmp.index(self._text)] = value
        self.b.children = []
        self.b.children = tmp
        self._text = value

    @property
    def dark(self):
        return self._dark

    @dark.setter
    def dark(self, flag):
        """
        Change the dark mode for the button

        Example
        -------
        Creation of a button and programmatically change its dark mode::

            from vois.vuetify import settings, Button

            b = Button('Test button', text_weight=450, width=250, height=46,
                       selected=True, rounded=True)
            display(b)
            b.dark = True

        """
        self._dark = flag
        
        self.icon_color = 'black'
        if self._dark:
            self.icon_color = 'white'
        
        self.b.dark = self._dark
        
        for item in self.b.children:
            if type(item).__name__ == 'Icon':
                newicon = v.Icon(class_="pa-0 ma-0 %s" % self.icon_distance, large=self.icon_large,
                                 small=self.icon_small,
                                 color=self.icon_color, children=[self._icon])
                self.b.children = [newicon if x == item else x for x in self.b.children]
                break
        
    @property
    def rounded(self):
        return self._dark

    @rounded.setter
    def rounded(self, flag):
        """
        Change the rounded state for the button

        Example
        -------
        Creation of a button and programmatically change its rounded state::

            from vois.vuetify import settings, Button

            b = Button('Test button', text_weight=450, width=250, height=46,
                       selected=True, rounded=True)
            display(b)
            b.rounded = False

        """
        self._rounded = flag
        self.b.rounded = self._rounded
        


"""Initial page for an application"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from vois.vuetify import settings, dialogGeneric, card
from ipywidgets import widgets, Layout, HTML
from IPython.display import display
import ipyvuetify as v
import os


######################################################################################################################################################
# Check if running in Voila: /eos is not mounted...
######################################################################################################################################################
def RunningInVoila():
    return not os.path.isdir("/eos/jeodpp/data/SRS/")


#####################################################################################################################################################
# Main page o the application
#####################################################################################################################################################
class mainPage():
    """
    Initial page of an application. The page contains a box for the title/subtitle/logo of the application, a box containing one or more buttons, each of them calling a callback function, and a box containing credits information. All these boxes are horizontally centered inside the page, while their vertical position can be customized (by default the title is on top, the buttons in central position and the credit box is at the bottom of the page).
        
    Parameters
    ----------
    title : str, optional
        Title of the page (default is 'Application main title')
    subtitle : str, optional
        Subtitle of the page (default is 'Subtitle to be displayed below the main title')
    applogo_url : str, optional
        Url of the application logo (default is a sample logo)
    applogo_widthpercent : float, optional
        Width of the area where the application logo is displayed in percentage of the screen (default is 35.0)
    credits : str, optional
        Credits string to display inside the credit box on the bottom of the page (default is 'Credits Team XXX')
    creditslogo_url : str, optional
        Url of the image to display inside the credit box on the bottom of the page (default is 'https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/TransparentJRC.png')
    text_color : str, optional
        Color to use for text (title, subtitle, credits, buttons text, etc.). Default is '#0e446e'.
    background_image : str or int, optional
        Image to use as fullscreen background. If an integer in the range [0,59] is passed, one of the sixty predefined wallpapers is used, otherwise any image URL can be used (default is 22)
    background_filter : str, optional
        CSS image filter to apply to the background image (default is ''). See https://developer.mozilla.org/en-US/docs/Web/CSS/filter for a list of available filters.
    vois_show : bool, optional
        Is True, a credit to the vois library is displayed in the top-right side of the page (default is True)
    vois_opacity : float, optional
        Opacity to apply to the vois logo, in case vois_show is set to True (default is 0.4)
    titlebox_widthpercent : float, optional
        Width of the top box containing the title, in percentage of the screen width. Default is 40.0.
    titlebox_heightpercent : float, optional
        Height of the box containing the title, in percentage of the screen height. Default is 24.0.
    titlebox_toppercent : float, optional
        Top position of the box containing the title, in percentage of the screen height, measured from the top of the page. Default is 14.0.
    titlebox_opacity : float, optional
        Opacity to apply to the box containing the title (default is 0.4)
    titlebox_border : int, optional
        Border width in pixels of the box containing the title (default is 4)
    buttonbox_widthpercent : float, optional
        Width of the box containing the buttons, in percentage of the screen width. Default is 80.0.
    buttonbox_heightpercent : float, optional
        Height of the box containing the buttons, in percentage of the screen height. Default is 50.0.
    buttonbox_toppercent : float, optional
        Top position of the box containing the buttons, in percentage of the screen height, measured from the top of the page. Default is 43.0.
    creditbox_widthpercent : float, optional
        Width of the box containing the credits, in percentage of the screen width. Default is 80.0.
    creditbox_heightpercent : float, optional
        Height of the box containing the credits, in percentage of the screen height. Default is 17.0.
    creditbox_toppercent : float, optional
        Top position of the box containing the credits, in percentage of the screen height, measured from the top of the page. Default is 80.0.
    creditbox_opacity : float, optional
        Opacity to apply to the box containing the credits (default is 0.6)
    button_widthpercent : float, optional
        Width of each of the buttons, in percentage of the screen width. Default is 30.0.
    button_heightpercent : float, optional
        Height of each of the buttons, in percentage of the screen height. Default is 10.0.
    button_elevation : int, optional
        Elevation in pixels to apply to the buttons (default is 6)
    button_opacity: float, optional
        Opacity to apply to the buttons (default is 0.5)
    disclaimer: str, optional
        Text to display at the bottom of the creditbox (default is the empty string)

    Examples
    --------
    Creation of a main page with 6 buttons displaying random images from https://picsum.photos/::
        
        from vois.vuetify import mainPage
        from random import randrange
        
        def onclick1():
            print("Clicked Function 1")

        m = mainPage.mainPage(title='mainPage demo',
                              subtitle='Showcase how easy is to create a front page for an app',
                              credits="vois library development team",
                              titlebox_widthpercent=50, titlebox_opacity=0.2, titlebox_border=0,
                              vois_show=True, vois_opacity=0.1,
                              button_widthpercent=23, button_heightpercent=14, button_elevation=16, button_opacity=0.6,
                              background_image=55,
                              background_filter='blur(2px) brightness(1.2) contrast(0.7) sepia(0.05) saturate(1.2)',
                              creditbox_opacity=0,
                              text_color='#222222')

        m.addButton('Function 1',
                    subtitle='Launch calculation of ...',
                    tooltip='Tooltip text to display on hover',
                    image='https://picsum.photos/seed/%d/200/200'%randrange(1000),
                    onclick=onclick1)

        for i in range(2,7): m.addButton('Function %d'%i, image='https://picsum.photos/seed/%d/200/200'%randrange(1000))

        m.open()


    .. figure:: figures/mainPage.png
       :scale: 100 %
       :alt: mainPage widget

       Example of a mainPage
       

    """

    # Initialization
    def __init__(self,
                 title='Application main title',
                 subtitle='Subtitle to be shown below the main title',
                 applogo_url='https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/sampleapplogo.png',
                 applogo_widthpercent=35.0,
                 credits='Credits Team XXX',
                 creditslogo_url='https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/TransparentJRC.png',
                 text_color='#0e446e',
                 background_image=22,
                 background_filter='',
                 vois_show=True,
                 vois_opacity=0.4,
                 titlebox_widthpercent=40.0,
                 titlebox_heightpercent=24.0,
                 titlebox_toppercent=14.0,
                 titlebox_opacity=0.4,
                 titlebox_border=4,
                 buttonbox_widthpercent=80.0,
                 buttonbox_heightpercent=50.0,
                 buttonbox_toppercent=43.0,
                 creditbox_widthpercent=80.0,
                 creditbox_heightpercent=17.0,
                 creditbox_toppercent=80.0,
                 creditbox_opacity=0.6,
                 button_widthpercent=30.0,
                 button_heightpercent=10.0,
                 button_elevation=6,
                 button_opacity=0.5,
                 disclaimer=''):

        self.output = widgets.Output(layout=Layout(width='0px', height='0px'))
        
        self.title                   = title
        self.subtitle                = subtitle
        self.applogo_url             = applogo_url
        self.applogo_widthpercent    = applogo_widthpercent
        self.credits                 = credits
        self.creditslogo_url         = creditslogo_url
        self.text_color              = text_color
        self.background_image        = background_image
        self.background_filter       = background_filter
        self.vois_show               = vois_show
        self.vois_opacity            = vois_opacity

        self.titlebox_widthpercent   = titlebox_widthpercent
        self.titlebox_heightpercent  = titlebox_heightpercent
        self.titlebox_toppercent     = titlebox_toppercent
        self.titlebox_opacity        = titlebox_opacity
        self.titlebox_border         = titlebox_border
        
        self.buttonbox_widthpercent  = buttonbox_widthpercent
        self.buttonbox_heightpercent = buttonbox_heightpercent
        self.buttonbox_toppercent    = buttonbox_toppercent
        
        self.creditbox_widthpercent  = creditbox_widthpercent
        self.creditbox_heightpercent = creditbox_heightpercent
        self.creditbox_toppercent    = creditbox_toppercent
        self.creditbox_opacity       = creditbox_opacity
        
        self.button_widthpercent     = button_widthpercent
        self.button_heightpercent    = button_heightpercent
        self.button_elevation        = button_elevation
        self.button_opacity          = button_opacity
        
        self.disclaimer              = disclaimer
        
        self.buttons = []

        
        # Spacers
        self.spacer = v.Html(tag='div',children=[' '], style_='width: 16px; height: 20px;')

        # Dimension of the main title rectangle
        width = '%fvw'%self.titlebox_widthpercent
        height = '%fvh'%self.titlebox_heightpercent


        
        ######################################################################################################################################################
        # Main titles
        ######################################################################################################################################################
        html_main = '''
        <head>
            <meta charset="utf-8" />
            <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
            <style>
                .big {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(5.0vh, 3.7vw);
                    font-weight: 700;
                    text-shadow: -0.06vh -0.06vh 0 #fff, 0.06vh -0.06vh 0 #fff, -0.06vh 0.06vh 0 #fff, 0.06vh 0.06vh 0 #fff;
                    text-align: center;
                    line-height: 2.0;
                    color: %s;
                }
                .normal {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(2.65vh, 2.0vw);
                    font-weight: 700;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                }
                .responsive4 {
                    width: %fvh;
                    vertical-align: middle;
                    height: auto;
                }
            </style>
        </head>
        <body>
            <div class="big">
                %s
            </div>

            <div class="normal">
                %s
            </div>

            <div class="normal">
                <br/>
                <img src="%s" alt="" class="responsive4">
            </div>
        </body>
        ''' % (self.text_color, self.text_color, self.applogo_widthpercent, self.title, self.subtitle, self.applogo_url)

        cardmain = v.Card(flat=True, children=[HTML(html_main)], style_="background-color: #ffffff00;")

        bcol = '#ffffff%0.2X'%(int(self.titlebox_opacity*255))
        self.hmain = v.Html(tag='div', children=[cardmain], class_="pa-0 ma-0",
                            style_='width: %s; height: %s; border: %dpx solid %s; border-radius: 12px; position: absolute; top: %fvh; left: %fvw; background-color: %s; overflow: hidden;'%(width,height, self.titlebox_border, self.text_color,
                                                                                                                                                                                           self.titlebox_toppercent, (100.0-self.titlebox_widthpercent)/2.0,
                                                                                                                                                                                           bcol))


        ######################################################################################################################################################
        # Credit to vois
        ######################################################################################################################################################
        if self.vois_show:
            html_vois = '''
            <head>
                <meta charset="utf-8" />
                <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
                <style>
                    .normal2 {
                        font-family: 'PT Sans', sans-serif;
                        font-size: min(2.20vh, 1.5vw);
                        font-weight: 700;
                        text-align: center;
                        line-height: 1.0;
                        color: %s;
                    }
                    .responsive2 {
                        width: 10vw;
                        vertical-align: middle;
                        height: auto;
                        background-color: #ffffff%0.2X;
                        padding: 10px;
                    }
                </style>
            </head>
            <body>
                <div class="normal2">
                    <br/>
                    <span style="">created using the: </span>
                    <img src="https://jeodpp.jrc.ec.europa.eu/services/shared/pngs/vois_horizontal_big.png" alt="" class="responsive2">
                </div>
            </body>
            ''' % (self.text_color, int(self.vois_opacity*255))

            cardvois = v.Card(flat=True, children=[HTML(html_vois)], style_="background-color: #ffffff00;")

            self.hvois = v.Html(tag='div', children=[cardvois], class_="pa-0 ma-0",
                                style_='width: 34vw; height: 11vh; border: 0px solid #000000; border-radius: 12px; position: absolute; top: 1vh; left: 65vw; background-color: #ffffff00; overflow: hidden;')
        else:
            self.hvois = ''



        ######################################################################################################################################################
        # Logo and credits
        ######################################################################################################################################################
        html_credits = '''
        <head>
            <meta charset="utf-8" />
            <link href='http://fonts.googleapis.com/css?family=PT+Sans' rel='stylesheet' type='text/css'>
            <style>
                .normal3 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(2.0vh, 1.4vw);
                    font-weight: 700;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                    overflow: hidden;
                }
                .normal4 {
                    font-family: 'PT Sans', sans-serif;
                    font-size: min(1.3vh, 0.9vw);
                    font-weight: 400;
                    text-align: center;
                    line-height: 1.0;
                    color: %s;
                    overflow: hidden;
                }
                .responsive3 {
                    width: 14vw;
                    vertical-align: middle;
                    height: auto;
                }
            </style>
        </head>
        <body>
            <div class="normal3">
                <img src="%s" alt="" class="responsive3">
            </div>
            <div class="normal3">
                <br/>
                %s
            </div>
            <div class="normal4">
                <br/>
                %s
            </div>
        </body>
        '''%(self.text_color, self.text_color, self.creditslogo_url, self.credits, self.disclaimer)

        cardunit = v.Card(flat=True, children=[HTML(html_credits)], style_="background-color: #ffffff00;", class_="pa-0 ma-0 mt-2")
        bcol = '#ffffff%0.2X'%(int(self.creditbox_opacity*255))
        self.hunit = v.Html(tag='div', children=[cardunit], class_="pa-0 ma-0",
                            style_='width: %fvw; height: %fvh; border: 0px solid #000000; border-radius: 12px; position: absolute; top: %fvh; left: %fvw; background-color: %s; overflow: hidden;'%(self.creditbox_widthpercent, self.creditbox_heightpercent,
                            self.creditbox_toppercent, (100.0-self.creditbox_widthpercent)/2.0, bcol))



    # Add a button
    def addButton(self, title, subtitle='', tooltip='', image='', onclick=None, argument=None):
        """
        Add a button to the page
        
        Parameters
        ----------
        title : str
            Title of the button
        subtitle : str, optional
            Subtitle of the button (default is '')
        tooltip : str, optional
            Tooltip to show when hovering the button title (default is '')
        image : str, optional
            Image to show on the right side of the button (default is '')
        onclick : function, optional
            Python function to call when the user clicks on the button. The function will receive the argument value as parameter if it is not None, otherwise it will be calle with no parameters. Default is None
        argument : any, optional
            Argument to pass to the onclick python function (default is None)
            
        """
        self.buttons.append({'title':    title,
                             'subtitle': subtitle,
                             'tooltip':  tooltip,
                             'image':    image,
                             'onclick':  onclick,
                             'argument': argument})
        
        
    # Display the page as a fullscreen dialog-box
    def open(self):
        """Open the page"""
        nbuttons_per_row = self.buttonbox_widthpercent // self.button_widthpercent
        
        rows = []
        
        children = []
        i = 0
        for b in self.buttons:
            c = card.card(elevation=self.button_elevation, width='%fvw'%self.button_widthpercent, imagesize="%fvh"%self.button_heightpercent, responsive=True,
                          title=b['title'], subtitle=b['subtitle'], titletooltip=b['tooltip'],
                          color='#ffffff%0.2X'%int(self.button_opacity*255), fontsizemultiplier=1.1, subtitleweight=400,
                          image=b['image'], on_click=b['onclick'], argument=b['argument'], textcolor=self.text_color)
            if len(children) > 0:
                children.append(self.spacer)
            children.append(c)
            
            i += 1
            if i >= nbuttons_per_row:
                r = v.Row(justify="center", children=children, class_="pa-0 ma-0 mt-2")
                rows.append(r)
                i = 0
                children = []
                
        if len(children) > 0:
            r = v.Row(justify="center", children=children, class_="pa-0 ma-0 mt-2")
            rows.append(r)
            
        hpages = v.Html(tag='div', children=rows, class_="pa-0 ma-0",
                        style_='width: %fvw; height: %fvh; position: absolute; top: %fvh; left: %fvw; background-color: #ffffff00; overflow: hidden;'%(self.buttonbox_widthpercent, self.buttonbox_heightpercent, self.buttonbox_toppercent, (100.0-self.buttonbox_widthpercent)/2.0))

        ######################################################################################################################################################
        # Background image
        ######################################################################################################################################################
        if isinstance(self.background_image, int):
            url = 'https://jeodpp.jrc.ec.europa.eu/services/shared/wallpapers/%d.jpg'%self.background_image
        else:
            url = self.background_image

        # See: https://css-tricks.com/perfect-full-page-background-image/
        self.back = v.Html(tag='div', children=[], class_="pa-0 ma-0",
                           style_='''
        width: 100vw; height: 100vh; 
        background-image: url('%s'); no-repeat center center fixed;
        -webkit-background-size: cover;
        -moz-background-size: cover;
        -o-background-size: cover;
        background-size: cover;
        filter: %s;
        '''%(url,self.background_filter))

        display(self.output)
        
        self.dlg = dialogGeneric.dialogGeneric(title='', titleheight='0px', text='', show=True, addclosebuttons=False, 
                                               persistent=RunningInVoila(), no_click_animation=True,    # In JEO-lab the page closes with the "ESC" key!!!
                                               fullscreen=True, content=[self.back, self.hmain, self.hvois, hpages, self.hunit], output=self.output)
        
        
    # Close the dialog
    def close(self):
        """Close the page"""
        self.page = None
        if not self.dlg is None:
            self.dlg.close()



"""SVG RankChart to display vertically aligned rectangles."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from ipywidgets import HTML, widgets, Layout
from ipyevents import Event

import statistics

try:
    from . import colors
    from .vuetify import fontsettings
except:
    import colors
    from vuetify import fontsettings

    
    
###########################################################################################################################################################################
# Display labels ranked by numerical value and manage selection by click using ipyevents
# All horizontal measures are in vw units, all vertical measures are in vh units
###########################################################################################################################################################################
def svgRankChart(title='Ranking of labels',
                 width=20.0,
                 height=90.0,
                 names=[],
                 values=[],
                 splitnamelenght=40,
                 addposition=True,
                 fontsize=1.1,
                 titlecolor='black',
                 textweight=400,
                 colorlist=['rgb(247,251,255)', 'rgb(222,235,247)', 'rgb(198,219,239)', 'rgb(158,202,225)', 'rgb(107,174,214)','rgb(66,146,198)', 'rgb(33,113,181)', 'rgb(8,81,156)', 'rgb(8,48,107)'], # Blues inverted
                 fixedcolors=False,
                 enabledeselect=False,
                 selectfirstatstart=True,
                 selectcolor='red',
                 hovercolor='yellow',
                 stdevnumber=2.0,             # Number of stddev to calculate (minvalue,maxvalue) range
                 minallowed_value=None,       # Minimum value allowed
                 maxallowed_value=None,       # Maximum value allowed
                 on_change=None):             # Function to call when the selected name is changed
    """
    Creation of a chart given a list of labels and corresponding numerical values. The labels are ordered according to the decreasing values and displayed as a vertical list of rectangles. Click on the rectangles is managed by calling a custom python function.
    
    Parameters
    ----------
    title : str, optional
        Title of the chart (default is 'Ranking of labels')
    width : float, optional
        Width of the chart in vw units (default is 20.0)
    height : float, optional
        Height of the chart in vh units (default is 90.0)
    names : list of str, optional
        List of names to display inside the rectangles (default is [])
    values : list of float, optional
        List of numerical values of the same length of the names list (default is [])
    splitnamelenght : int, optional
        Maximum number of characters to display in a row. If the name length is higher, it is splitted in two rows (default is 40)
    addposition : bool, optional
        If True, the position is added in front of the name (starting from 1 and determined by the accompanying value). Default is True
    fontsize : float, optional
        Size of the standard font to use for names in vh coordinates (default is 1.1vh). The chart title will be displayed with sizes proportional to the fontsize parameter (up to two times for the chart title)
    titlecolor : str, optional
        Color to use for the chart title (default is 'black')
    textweight : int, optional
        Weight of the text written inside the rectangles (default is 400). The chart title will be displayed with weight equal to textweight+100
    colorlist : list of colors, optional
        List of colors to assign to the rectangles based on the numerical values (default is the inverted Plotly px.colors.sequential.Blues, see `Plotly sequential color scales <https://plotly.com/python/builtin-colorscales/#builtin-sequential-color-scales>`_ and `Plotly qualitative color sequences <https://plotly.com/python/discrete-color/#color-sequences-in-plotly-express>`_ )
    fixedcolors : bool, optional
        If True, the list of colors is assigned to the values in their original order (and colorlist must contain the same number of elements!). Default is False
    enabledeselect : bool, optional
        If True, a click on a selected element deselects it, and the on_change function is called with None as argument (default is False)
    selectfirstatstart : bool, optional
        If True, at start the rectangle corresponding to the greatest value is selected (default is True)
    selectcolor : str, optional
        Color to use for the border of the selected rectangle (default is 'red')
    hovercolor : str, optional
        Color to use for the border on hovering the rectangles (default is 'yellow')
    stdevnumber : float, optional
        The correspondance between the values and the colors list is done by calculating a range of values [min,max] to linearly map the values to the colors. This range is defined by calculating the mean and standard deviation of the values and applying this formula [mean - stdevnumber*stddev, mean + stdevnumber*stddev]. Default is 2.0
    minallowed_value : float, optional
        Minimum value allowed, to force the calculation of the [min,max] range to map the values to the colors
    maxallowed_value : float, optional
        Maximum value allowed, to force the calculation of the [min,max] range to map the values to the colors
    on_change: function, optional
        Python function to call when the selection of the rectangle items changes (default is None). The function is called with a tuple as unique argument. The tuple will contain (name, value, originalposition) of the selected rectangle
            
    Returns
    -------
        an instance of widgets.Output with the svg chart displayed in it

    Example
    -------
    Creation of a SVG chart to display some names ordered by correspondent values::

        from IPython.display import display
        from ipywidgets import widgets
        from vois import svgRankChart
        import numpy as np
        import plotly.express as px

        # List of sentences
        names = ['European society needs to grasp the opportunities brought by the digital transformation', 
                 'A deep transformation such as the one facilitated by digital technologies',
                 'The progress needs to be evenly distributed across all regions',
                 'Over the next decade the EU economy and society need to undergo a profound transformation',
                 'The design of green and digital policy actions needs to consider socio-economic and territorial impacts',
                 'The EU supports the shift to a sustainable and resilient growth model',
                 'Address the challenges arising from the demographic transition',
                 'Geospatial data and methods are (usually) globally applicable',
                 'Considerable EU investments',
                 'The acceleration of the implementation of the Fit for 55 package',
                 'To become climate-neutral by 2050, Europe needs to decarbonise',
                 'Efforts need to be intensified in the harder-to-decarbonise sectors',
                 'The Commission recently decided to better understand the environment interface',
                 'One Health was already recognised by the Commission as an emerging priority',
                 'Achieving sustainability requires a holistic, well-coordinated approach',
                 'The conflict in Ukraine is endangering food security and has implications for food supply chains',
                 'The food system is composed of sub-systems and interacts with other key systems',
                 'The future of the EU and its position in the world will be influenced by population trajectories',
                 'Policymakers are often asked to react quickly to new circumstances',
                 'Obtaining economic benefit from natural resources whilst maintaining natural capital',
                 'We must be able to create EU policies that  decouple resource use from economic development',
                 'Robust, resilient and innovative EU economy is a necessary condition for ensuring the well-being'
                 ]

        # Randomly generated values for each sentence
        values = np.random.uniform(low=0.5, high=25.0, size=(len(names,)))

        debug = widgets.Output()
        display(debug)

        def on_change(arg):
            with debug:
                print(arg)

        out = svgRankChart.svgRankChart(names=names,
                                        values=values,
                                        width=20.0,
                                        height=90.0,
                                        splitnamelenght=45,
                                        addposition=False,
                                        fontsize=1.3,
                                        selectfirstatstart=False,
                                        colorlist=px.colors.sequential.Viridis,
                                        hovercolor='blue',
                                        on_change=on_change)
        display(out)
        
    .. figure:: figures/rankchart.png
       :scale: 100 %
       :alt: svgRankChart example

       Example of an interactive and ordered list of rectangles
        
    """

    if len(names) == 0:
        return None
    
    if len(names) != len(values):
        print("Names and values lists have different number of elements!")
        return None
    
    if fixedcolors and len(names) != len(colorlist):
        print("Names and colorlist lists have different number of elements!")
        return None
    
    
    # Returns 2 strings from 1
    def splitString(s):
        pos = len(s)//2
        spaces = [i for i, ltr in enumerate(s) if ltr == ' ']
        if len(spaces) > 0:
            bestpos = min(spaces, key=lambda x:abs(x-pos))
            s1 = s[:bestpos]
            s2 = s[bestpos+1:]
            return s1,s2
        return s,''
    
    fontsize *= 3.0
    svgwidth = 100.0
    aspectratio = 0.5*height / width   # In landscape mode, usually the height is half the width dimension!!!
    svgheight = svgwidth * aspectratio
    
    titlefontsize = fontsize * 1.75

    hTitle = 1.2*titlefontsize
    helem = (svgheight-hTitle)/len(names)
    x = 0.1
    y = hTitle

    name2color = {}
    if fixedcolors:
        name2color.update(zip(names,colorlist))
        
        
    numbers = range(1,len(names)+1)
    ordered = sorted(zip(names,values,numbers), key=lambda x: -x[1]) # Reverse order
    
    selected = -1
    if selectfirstatstart: 
        selected = ordered[0][2]

    #preserve = 'none'
    preserve = 'xMidYMid meet'
    
    def createSVG():
        svg = '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xml:space="preserve" viewBox="0 0 %f %f" preserveAspectRatio="%s" width="%fvw" height="%fvh">' % (svgwidth,svgheight, preserve, width,height)

        svg += '''
    <style type="text/css">
         @import url('%s');
         .prio:hover {cursor: pointer; stroke-width: 1.0; stroke: %s; }
    </style>     
    ''' % (fontsettings.font_url,hovercolor)
    
        mean = statistics.mean(values)
        if len(names) <= 1:
            minvalue = mean
            maxvalue = mean
        else:
            stddev = statistics.stdev(values)
            valuemin = min(values)
            valuemax = max(values)

            minvalue = mean - stdevnumber*stddev
            maxvalue = mean + stdevnumber*stddev

            if minvalue < valuemin: minvalue = valuemin
            if maxvalue > valuemax: maxvalue = valuemax

        if not minallowed_value is None:
            if minvalue < minallowed_value: minvalue = minallowed_value
        if not maxallowed_value is None:
            if maxvalue > maxallowed_value: maxvalue = maxallowed_value

        if minvalue >= maxvalue: maxvalue = minvalue + 1
        ci = colors.colorInterpolator(colorlist,minvalue,maxvalue)

        svg += '<text x="%f" y="%f" text-anchor="middle" font-family="%s" font-size="%f" fill="%s" font-weight="%d">%s</text>' % (svgwidth/2.0, 2.2*titlefontsize/3.0, fontsettings.font_name, titlefontsize, titlecolor, textweight+100, title)
        x = 0.1
        y = hTitle

        i = 0
        for name,value,pos in ordered:
                
            if fixedcolors:
                col = name2color[name]
            else:
                col = ci.GetColor(value)
                
            if colors.isColorDark(colors.string2rgb(col)): textcol = 'white'
            else:                                          textcol = 'black'

            if pos==selected:
                strokew = 1.0
                stroke  = selectcolor
            else:
                strokew = 0.2
                stroke  = 'black'

            spos = ''
            if addposition: spos = str(pos) + '. '
                
            svg += '<rect class="prio" x="%f" y="%f" width="%f" height="%f" fill="%s" stroke-width="%f" stroke="%s"><title>%s%s: %d%%</title></rect>' % (x, y, svgwidth-0.2, helem-0.5, col, strokew, stroke, spos, name, int(value*100.0+0.5))
            if len(name) >= splitnamelenght:
                s1,s2 = splitString(name)
                svg += '<text style="pointer-events: none" x="%f" y="%f" text-anchor="middle" font-family="%s" font-size="%f" fill="%s" font-weight="%d">%s%s</text>' % (svgwidth/2, y+fontsize, fontsettings.font_name, fontsize, textcol, textweight, spos, s1)
                svg += '<text style="pointer-events: none" x="%f" y="%f" text-anchor="middle" font-family="%s" font-size="%f" fill="%s" font-weight="%d">%s</text>' % (svgwidth/2, y+2*fontsize, fontsettings.font_name, fontsize, textcol, textweight, s2)
            else:
                svg += '<text style="pointer-events: none" x="%f" y="%f" text-anchor="middle" font-family="%s" font-size="%f" fill="%s" font-weight="%d">%s%s</text>' % (svgwidth/2, y+1.5*fontsize, fontsettings.font_name, fontsize, textcol, textweight, spos, name)
            y += helem
            i += 1

        svg += '</svg>'
        return svg
    
    # Pixels to add to the output Widget in order to not see the scrollbars
    added_pixels_width  = 20
    added_pixels_height = 20
    
    # Create an output widget and display SVG in it
    out = widgets.Output(layout=Layout(width='calc(%fvw + %dpx)'%(width,added_pixels_width), height='calc(%fvh + %dpx)'%(height,added_pixels_height), margin='0px 0px 0px 0px')) #border='1px dashed green'))

    svg = createSVG()
    with out:
        display(HTML(svg))

    # Add an event manager to the out Output widgets
    d = Event(source=out, watched_events=['click'])

    #debug = widgets.Output()

    # Manage click event
    def handle_event(event):
        nonlocal selected
        x = event['relativeX']
        y = event['relativeY']
        w = event['boundingRectWidth']  - added_pixels_width
        h = event['boundingRectHeight'] - added_pixels_height
        ar = float(h) / float(w)
        
        # Calculate dimension and bounding box of chart in pixels
        if ar > aspectratio:
            chartw = w
            charth = w * aspectratio
            xmin   = 0
            xmax   = chartw
            ymin   = (h - charth)/2
            ymax   = ymin + charth
        else:
            charth = h
            chartw = h / aspectratio
            xmin   = (w - chartw)/2
            xmax   = xmin + chartw
            ymin   = 0
            ymax   = charth

        # Calculate position in percentage on the chart area in pixels
        if y >= ymin and y <= ymax:
            yp = 100.0 * (y-ymin) / float(charth) - 1.0
            hTitlePercent = 100.0*hTitle/svgheight
            hrect = (100.0 - hTitlePercent)/float(len(names))
            #with debug:
            #    print(yp,hTitlePercent,hrect)
            
            if yp >= hTitlePercent*0.9:
                elem = ordered[int((yp-hTitlePercent)//hrect)]
                #with debug:
                #    print(yp, hTitlePercent, helem, elem)
        
                if enabledeselect:
                    if elem[2] == selected:
                        selected = -1
                    else:
                        selected = elem[2]
                else:
                    selected = elem[2]
                out.clear_output(wait=True)
                with out:
                    svg = createSVG()
                    display(HTML(svg))

                if not on_change is None:
                    if selected < 0:
                        on_change(None)
                    else:
                        on_change(elem)   # Tuple containing (name, value, originalposition)
    
    d.on_dom_event(handle_event)

    if selectfirstatstart and (not on_change is None):
        on_change(ordered[0])   # Tuple containing (name, value, originalposition)
    
    return out#, debug


"""Upload local json file: dialog box to select and previw of some fields"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
from ipywidgets import widgets, Layout
import ipyvuetify as v

# Vois imports
from vois import cssUtils, colors
from vois.vuetify import settings, upload, dialogGeneric, dialogMessage, dialogWait

# Python imports
import json


# Creation of a label
def label(text, class_='pa-0 ma-0 mt-1 mr-3', size=14, weight=400, color=settings.color_first, width=None):
    lab = v.Html(tag='div', children=[text], class_=class_)
    if width is None:
        lab.style_ = 'font-size: %dpx; font-weight: %d; color: %s;'%(size,weight,color)
    else:
        lab.style_ = 'font-size: %dpx; font-weight: %d; color: %s; width: %dpx'%(size,weight,color,int(width))
    return lab


#####################################################################################################################################################
# Upload local json file: dialog box to select and previw of some fields
#####################################################################################################################################################
class UploadJson():
    
    # Initialization
    def __init__(self,
                 output,
                 message='Click to select the file to upload',
                 label='File json:',
                 accept='application/json',
                 title='Select a file',
                 required_attributes=[],         # List of attribute names to check in selected json file and to display in the preview
                 attributes_width=100,           # width in pixel of the spavce to display the required attributes names
                 color=settings.color_first,
                 dark=settings.dark_mode,
                 titleheight=40,
                 width=800,
                 onOK=None,                      # Called passing the content of the selected file
                 onCancel=None):                 # Called with no argument
        
        self.output              = output
        self.title               = title
        self.required_attributes = required_attributes
        self.attributes_width    = attributes_width
        self.color               = color
        self.dark                = dark
        self.titleheight         = titleheight
        self.width               = width
        self.onOK                = onOK
        self.onCancel            = onCancel
        
        cssUtils.allSettings(self.output)
        
        self.json = None
        
        self.wait = None
        
        self.preview = widgets.Output(layout=Layout(height='200px'))
        self.u = upload.upload(accept=accept, label=label, onchanging=self.onFileSelected, onchange=self.onFileUpload, placeholder=message, multiple=False, color=self.color)

        spacerY = v.Html(tag='div', style_='width: 0px; height: 20px;')
        self.content = v.Card(flat=True, class_='pa-0, ma-0 mt-n4 ml-4 mr-4', children=[widgets.VBox([self.u.draw(), spacerY, self.preview])])
        
        
    # Open the dialog-box
    def show(self):
        self.preview.clear_output()
        self.u.clear()
        self.dlg = dialogGeneric.dialogGeneric(title=self.title, on_ok=self._internal_onOK, on_cancel=self._internal_onCancel, on_close=self.onCancel,
                                               text='   ', color=self.color, dark=self.dark, titleheight=self.titleheight, width=self.width,
                                               show=True, addclosebuttons=True, addokcancelbuttons=True,
                                               fullscreen=False, content=[self.content], output=self.output)
        self.dlg.okdisabled = True
        
        
    # Just after the file is selected
    def onFileSelected(self):
        self.wait = dialogWait.dialogWait(text='Uploading file...', output=self.output, color=self.color, dark=self.dark)

        
    # Selection of an image to upload
    def onFileUpload(self, files):
        # If at least one file has been selected      
        if len(files) > 0:
            
            self.preview.clear_output()
            for f in files:
                self.json = json.load(f['file_obj'])
                
                # Check the presence of the required attributes
                attributesOK = True
                for a in self.required_attributes:
                    if a not in self.json:
                        attributesOK = False
                        break
                        
                if attributesOK:
                    self.dlg.okdisabled = False
                    with self.preview:
                        for a in self.required_attributes:
                            display(widgets.HBox([label(a+':', color='black', width=self.attributes_width), label(self.json[a], color='black', weight=600)]))
                        
                else:
                    dialogMessage.dialogMessage(title='Error',
                                                text='The uploaded file has no %s attribute'%a,
                                                addclosebuttons=False, show=True, width=400, output=self.output)
                    
                    self.dlg.okdisabled = True
                    self.json = None
                    self.preview.clear_output()
                    
        # No files selected
        else:
            self.json = None
            self.preview.clear_output()

        if self.wait is not None:
            self.wait.close()
    
    
    # Selection done
    def _internal_onOK(self):
        if self.onOK is not None:
            self.onOK(self.json)
    
    # Exit without selection
    def _internal_onCancel(self):
        if self.onCancel is not None:
            self.onCancel()



"""Select widget that displays images and enables for single selection.""" 
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
import ipyvuetify as v
import traitlets 

try:
    from . import settings
except:
    import settings

    
class selectImage(v.VuetifyTemplate):
    """
    Select widget that displays images and enables for single selection.
        
    Parameters
    ----------
    images : list of json element, one for each image to display, optional
        Each of the json elements must have "name", and "image" tags. Optional tags are "max_width", "max_height", "margins"
    label : str, optional
        Label to show in the select widget (default is '')
    selection : int, optional
        Index of the image to display as selected in the selection widgets at start (default is -1)
    onchange : function, optional
        Python function to call when the user selects one of the images. The function will receive no parameters. (default is None)
    color : str, optional
        Main color of the selection widget (default is settings.color_first)
    outlined : bool, optional
        If True the selection widget will have a border around it (default is True)
    clearable : Bool, optional
        If True the selection widget will show a -x- button to clear the selection (default is True)
    width : str, optional
        Width of the select widget. It can be expressed as pixels (ex: "400px") or percentage (ex: "50%"). Default is "100%"
    dense : Bool, optional
        If True the items of the widgets have a reduced height (default is True)
    max_width : int, optional
        Max width in pixels of the images displayed inside the selection widget (default is 100)
    max_height : int, optional
        Max height in pixels of the images displayed inside the selection widget (default is 100)
    margins : str, optional
        Margins to apply to the images in the selection list and when displayed as selected in the widget (default is "ma-0 mr-1 mb-1")

    Example
    -------
    Creation of a select widget to select an image::
        
        from vois.vuetify import selectImage
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()

        def onchange():
            with output:
                print(s.value)


        images = [
              { "name": 'Image 0', "image": 'https://cdn.vuetifyjs.com/images/cards/plane.jpg'},
              { "name": 'Image 1', "image": 'https://cdn.vuetifyjs.com/images/cards/house.jpg'},
              { "name": 'Image 2', "image": 'https://cdn.vuetifyjs.com/images/cards/road.jpg'},
              { "name": 'Image 3', "image": 'https://cdn.vuetifyjs.com/images/cards/sunshine.jpg'}
        ]

        s = selectImage.selectImage(images=images, selection=1, max_height=100, onchange=onchange,
                                    label='Please select an image from the list', 
                                    outlined=True, clearable=True, margins="ma-1")

        display(s)
        display(output)
        
    .. figure:: figures/selectImage.png
       :scale: 100 %
       :alt: selectImage widget

       Example of a selectImage widget
    """
    
    images    = traitlets.Any([]).tag(sync=True)    # images to display (array of json objects containing "name", and "image" tags).
                                                    # Optional tags are "max_width", "max_height", "margins"
    selection = traitlets.Any(None).tag(sync=True)
    label     = traitlets.Unicode('').tag(sync=True)
    color     = traitlets.Unicode(settings.color_first).tag(sync=True)
    outlined  = traitlets.Bool(True).tag(sync=True)
    clearable = traitlets.Bool(True).tag(sync=True)
    dense     = traitlets.Bool(True).tag(sync=True)
    style     = traitlets.Unicode('width: %100%; max-width: 100%;').tag(sync=True)
    
    template  = traitlets.Unicode('''
<v-card style="overflow: hidden;">
  <v-select
    :value="selection"
    :v-model="selection"
    :items="images"
    :color="color"
    :clearable="clearable"
    :outlined="outlined"
    :label="label"
    class="pa-0 ma-0 mt-1 mb-n6"
    :dense="dense"
    :style="style"
    @input="input">

    <template v-slot:selection="{ item, index }">
        <v-img :class="item.margins" :src="item.image" :max-width="item.max_width"  :max-height="item.max_height" ></v-img>
        {{item.name}}
    </template>

    <template v-slot:item="{ item }">
        <v-img :class="item.margins" :src="item.image" :max-width="item.max_width"  :max-height="item.max_height" ></v-img>
        {{item.name}}
    </template>
  </v-select>
</template>

<style id="treeview-item-style">

.v-application .primary--text {
    color: #000000 !important;
    background-color: #ffffff !important;
    caret-color: #000000 !important;
}
.v-list .v-list-item--active { 
  background-color: #ffffff !important; 
}
.v-list .v-list-item--active .v-list-item__title {
  color: #000000 !important;
}

</style>
''').tag(sync=True)
    
    

    def __init__(self, images=[], label='', selection=-1, onchange=None,
                 color=settings.color_first, width="100%",
                 dense=True, outlined=True, clearable=True, 
                 max_width=100, max_height=100, margins="ma-0 mr-1 mb-1"):
        
        super().__init__()

        self.max_width  = max_width
        self.max_height = max_height
        self.margins    = margins
        
        i = 0
        for img in images:
            img["index"] = i
            i += 1
            if not "name" in img:
                img["name"] = ""
            if not "max_width" in img:
                img["max_width"] = str(self.max_width)
            if not "max_height" in img:
                img["max_height"] = str(self.max_height)
            if not "margins" in img:
                img["margins"] = self.margins
        
        self.images          = images
        self.label           = label
        self.onchange        = onchange
        self.color           = color
        self.dense           = dense
        self.outlined        = outlined
        self.clearable       = clearable
        self.style           = 'width: %s; max-width: %s;' % (str(width), str(width))
        self._value          = -1
        self.selection       = None
        if isinstance(selection, int) and selection >= 0 and selection < len(self.images):
            self._value = selection
            self.selection = self.images[selection]
        
        
    # Manage event "input"
    def vue_input(self, data):
        if 'index' in data:
            self._value = data['index']
        else:
            self._value = -1
        if self.onchange:
            self.onchange()

            
    # value property
    @property
    def value(self):
        """
        Get/Set the selected image index.
        
        Returns
        --------
        v : int
            index of the image currently selected

        Example
        -------
        Programmatically select one of the images given its index::
            
            sel.value = 3
            print(sel.value)
        
        """
        return self._value

    
    @value.setter
    def value(self, v):
        if isinstance(v, int) and v >= 0 and v < len(self.images):
            self._value = v
            self.selection = self.images[v]
        else:
            self._value = -1
            self.selection = ''
        if self.onchange: self.onchange()
            
            
    # Set the images
    def setImages(self, images):
        """
        Change the images to be selected.
        
        Parameters
        ----------
        images : list of json element, one for each image to display
            Each of the json elements must have "name", and "image" tags. Optional tags are "max_width", "max_height", "margins"
        """
        
        i = 0
        for img in images:
            img["index"] = i
            i += 1
            if not "name" in img:
                img["name"] = ""
            if not "max_width" in img:
                img["max_width"] = str(self.max_width)
            if not "max_height" in img:
                img["max_height"] = str(self.max_height)
            if not "margins" in img:
                img["margins"] = self.margins
        self.images = images
        
        self._value = -1
        self.selection = None
            


"""Widget to select a float value"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
from ipywidgets import widgets
import ipyvuetify as v

try:
    from . import settings
    from . import label
    from . import tooltip
except:
    import settings
    import label
    import tooltip


#####################################################################################################################################################
# sliderFloat control
#####################################################################################################################################################
class sliderFloat():
    """
    Compound widget to select a float value in a specific range.
        
    Parameters
    ----------
    value : float, optional
        Initial value of the slider (default is 1.0)
    minvalue : float, optional
        Minimum value of the slider (default is 0.0)
    maxvalue : float, optional
        Maximum value of the slider (default is 1.0)
    text : str, optional
        Text to display on the left of the slider (default is 'Select')
    showpercentage : bool, optional
        If True, the widget will write the current value as a percentage inside the allowed range and will append the % sign to the right of the current value (default is True)
    decimals : int, optional
        Number of decimal digits to use in case showpercentage is False (default is 2)
    maxint : int, optional
        Maximum integer number for the underlining integer slider (defines the slider sensitivity). Default value is None, meaning it will be automatically calculated
    labelwidth : int, optional
        Width of the label part of the widgets in pixels (default is 0, meaning it is automatically calculated based on the provided label text)
    sliderwidth : int, optional
        Width in pixels of the slider component of the widget (default is 200)
    resetbutton: bool, optional
        If True a reset button is displayed, allowing for resetting the widget to its initial value (default is False)
    showtooltip: bool, optional
        If True the up and down buttons will have a tooltip (default is False)
    onchange : function, optional
        Python function to call when the changes the value of the slider. The function will receive a single parameter, containing the new value of the slider in the range from minvalue to maxvalue (default is None)
            
    Example
    -------
    Creation and display of a slider widget to select an opacity value in the range [0.0, 1.0]::
        
        from vois.vuetify import sliderFloat
        from ipywidgets import widgets
        from IPython.display import display

        output = widgets.Output()

        def onchange(value):
            with output:
                print(value)

        o = sliderFloat.sliderFloat(0.8, text='Fill opacity:', minvalue=0.0, maxvalue=1.0, onchange=onchange)
        
        display(o.draw())
        display(output)

    .. figure:: figures/sliderFloat.png
       :scale: 100 %
       :alt: opacity widget

       Example of an slider widget to select a floating point value.
   """

    # Initialization
    def __init__(self, value=1.0, minvalue=0.0, maxvalue=1.0, text='Select', showpercentage=True, decimals=2, maxint=None, 
                 labelwidth=0, sliderwidth=200, resetbutton=False, showtooltip=False, onchange=None):
        
        self.onchange = onchange
        
        self.minvalue = minvalue
        self.maxvalue = maxvalue
        self.showpercentage = showpercentage
        self.decimals       = decimals
        
        if maxint is None:
            if self.decimals <= 1:
                self.maxint = 10
            elif self.decimals == 2:
                self.maxint = 100
            elif self.decimals == 3:
                self.maxint = 1000
            elif self.decimals == 4:
                self.maxint = 10000
            else:
                self.maxint = 100000
        else:
            self.maxint = maxint
        
        self.postchar = ''
        if self.showpercentage: self.postchar = '%'
            
        self.labelwidth  = labelwidth
        self.sliderwidth = sliderwidth
        self.resetbutton = resetbutton
        
        self.int_initial_value = self.float2integer(value)
        
        #self.label  = label.label(text, textweight=400, margins=0, margintop=4, height=22)
        
        if self.labelwidth > 0:
            spx = '%dpx'%self.labelwidth
            style = 'width: %s; min-width: %s; max-width: %s;'%(spx,spx,spx)
        else:
            style = ''
        self.label = v.Html(tag='div', children=[text], class_='pa-0 ma-0 mt-4', style_=style)
        self.slider = v.Slider(v_model=self.int_initial_value,
                               dense=True, xsmall=True, 
                               ticks=False, thumb_size=10, dark=settings.dark_mode,
                               color=settings.color_first, track_color="grey",
                               class_="pa-0 ma-0 ml-5 mr-4 mt-3 mb-n1",
                               style_='max-width: %dpx; width: %dpx;'%(self.sliderwidth,self.sliderwidth),
                               min=0, max=self.maxint, vertical=False, height=32)
        self.slider.on_event('input',  self.oninput)
        self.slider.on_event('change', self.onsliderchange)
        
        if self.showpercentage:
            self.labelvalue = v.Html(tag='div', children=[str(self.int_initial_value) + self.postchar], class_='pa-0 ma-0 mt-4')
        else:
            self.labelvalue = v.Html(tag='div', children=['{:.{prec}f}'.format(value, prec=self.decimals) + self.postchar], class_='pa-0 ma-0 mt-4')

        if self.resetbutton:
            self.bup = v.Btn(icon=True, small=True, rounded=False, elevation=0, width=15, height=20, class_='pa-0 ma-0', children=[v.Icon(color='grey', children=['mdi-menu-right'])])
            self.bup.on_event('click', self.onup)
            self.cup = v.Card(children=[self.bup], elevation=0, class_='pa-0 ma-0 mr-1 mt-4', style_="overflow: hidden;")
            if showtooltip: self.cup = tooltip.tooltip("Increase",self.cup)

            self.bdn = v.Btn(icon=True, small=True, rounded=False, elevation=0, width=15, height=20, class_='pa-0 ma-0', children=[v.Icon(color='grey', children=['mdi-menu-left'])])
            self.bdn.on_event('click', self.ondn)
            self.cdn = v.Card(children=[self.bdn], elevation=0, class_='pa-0 ma-0 mr-1 mt-4', style_="overflow: hidden;")
            if showtooltip: self.cdn = tooltip.tooltip("Decrease",self.cdn)

            self.bres = v.Btn(icon=True, x_small=True, rounded=False, elevation=0, width=15, height=15, class_='pa-0 ma-0', children=[v.Icon(color='grey', children=['mdi-close'])])
            self.bres.on_event('click', self.onreset)
            self.cres = v.Card(children=[self.bres], elevation=0, class_='pa-0 ma-0 mr-1 mt-4', style_="overflow: hidden;")
            if showtooltip: self.cres = tooltip.tooltip("Reset value",self.cres)
                
            self.buttons = widgets.HBox([self.cdn,self.cres,self.cup])
        else:
            self.bup = v.Btn(icon=True, small=True, rounded=False, elevation=0, width=15, height=20, class_='pa-0 ma-0', children=[v.Icon(color='grey', children=['mdi-menu-up'])])
            self.bup.on_event('click', self.onup)
            self.cup = v.Card(children=[self.bup], elevation=0, class_='pa-0 ma-0 mr-1 mt-2', style_="overflow: hidden;")
            if showtooltip: self.cup = tooltip.tooltip("Increase",self.cup)

            self.bdn = v.Btn(icon=True, small=True, rounded=False, elevation=0, width=15, height=20, class_='pa-0 ma-0', children=[v.Icon(color='grey', children=['mdi-menu-down'])])
            self.bdn.on_event('click', self.ondn)
            self.cdn = v.Card(children=[self.bdn], elevation=0, class_='pa-0 ma-0 mr-1 mt-n1', style_="overflow: hidden;")
            if showtooltip: self.cdn = tooltip.tooltip("Decrease",self.cdn)
            
            self.buttons = widgets.VBox([self.cup,self.cdn])
        
        
    # Input event on the slider
    def oninput(self, *args):
        if self.showpercentage:
            value = self.slider.v_model
            self.labelvalue.children=[str(value) + self.postchar]
        else:
            value = self.integer2float(self.slider.v_model)
            self.labelvalue.children=['{:.{prec}f}'.format(value, prec=self.decimals) + self.postchar]

            
    # Input change on the slider
    def onsliderchange(self, *args):
        if self.showpercentage:
            value = self.slider.v_model
            self.labelvalue.children=[str(value) + self.postchar]
        else:
            value = self.integer2float(self.slider.v_model)
            self.labelvalue.children=['{:.{prec}f}'.format(value, prec=self.decimals) + self.postchar]
        
        if not self.onchange is None:
            self.onchange(self.integer2float(self.slider.v_model))
        self.bup.disabled = self.slider.v_model >= self.maxint
        self.bdn.disabled = self.slider.v_model <= 0
            
            
    # Click on the up button
    def onup(self, *args):
        if self.slider.v_model < self.maxint:
            self.slider.v_model = self.slider.v_model + 1
            self.onsliderchange()
    
    # Click on the down button
    def ondn(self, *args):
        if self.slider.v_model > 0:
            self.slider.v_model = self.slider.v_model - 1
            self.onsliderchange()
    
    def onreset(self, *args):
        self.slider.v_model = self.int_initial_value
        self.onsliderchange()

    # Conversion of float value [minvalue,maxvalue] to integer in [0,self.maxint]
    def float2integer(self, value):
        return int(self.maxint * (max(self.minvalue, min(self.maxvalue, float(value))) - self.minvalue) / (self.maxvalue - self.minvalue))
    
    # Conversion of integer value [0,self.maxint] to float in [0.0,1.0]
    def integer2float(self, value):
        return ((self.maxvalue - self.minvalue) * float(value)) / float(self.maxint) + self.minvalue

    # Draw the widget
    def draw(self):
        """Returns the ipyvuetify object to display (a v.Row widget)"""
        return v.Row(justify='start', class_='pa-0 ma-0', no_gutters=True, children=[self.label, self.slider, self.buttons, self.labelvalue], style_="overflow: hidden;")

        
    # Get the slider value
    @property
    def value(self):
        """
        Get/Set the slider value.
        
        Returns
        --------
        value : float
            Current value of the slider

        Example
        -------
        Programmatically set the slider value
            
            s.value = 0.56
        
        """
        return self.integer2float(self.slider.v_model)
   

    # Set the slider value
    @value.setter
    def value(self, newvalue):
        if newvalue < self.minvalue:
            newvalue = self.minvalue
        if newvalue > self.maxvalue:
            newvalue = self.maxvalue
            
        intvalue = self.float2integer(newvalue)
        self.slider.v_model = intvalue
        self.onsliderchange()



"""Widget to display text strings vertically aligned."""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2022-2023
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.
from IPython.display import display
import ipyvuetify as v

try:
    from . import settings
except:
    import settings


#####################################################################################################################################################
# Testlist control
#####################################################################################################################################################
class textlist():
    """
    Widget to vertically display a list of titles and texts strings. Each couple of title and text occupies a row.
        
    Parameters
    ----------
    titles : list of strings
        Strings to be displayed as title of each row
    texts : list of strings
        Strings to be displayed as the content of each row
    titlesbold : list of strings, optional
        List of titles whose corresponding texts should be displayed in with bold font (default is [])
    titlefontsize : int, optional
        Size in pixel of the font used for the titles (default is 12)
    textfontsize : int, optional
        Size in pixel of the font used for the texts (default is 12)
    titlecolumn : int, optional
        Number of column (out of 12) occupied by the titles (default is 4)
    textcolumn : int, optional
        Number of column (out of 12) occupied by the texts (default is 8)
    titlecolor : str, optional
        Color to use for the titles (default is 'black')
    textcolor : str, optional
        Color to use for the texts (default is 'black')
    lineheightfactor : float, optional
        Factor to multiply to the font-size to calculate the height of each row (default is 1.5)

    Example
    -------
    Creation and display of a widget to display some textual information::
        
        from vois.vuetify import textlist
        from IPython.display import display
        
        t = textlist.textlist(['Name', 'Surname', 'Address', 'Role'],
                              ['Davide', 'De Marchi', 'via Eduardo 34, Roccacannuccia (PE)', 'Software developer'],
                              titlesbold=['Surname'],
                              titlefontsize=14,
                              textfontsize=16,
                              titlecolumn=3,
                              textcolumn=10,
                              titlecolor='#003300',
                              textcolor='#000000',
                              lineheightfactor=1.4
                             )

        # Set some attributes of the card widget (margins, colors, width, etc.)
        t.card.class_    = 'pa-0 ma-4 ml-6 mr-8'
        t.card.flat      = False
        t.card.color     = '#e0ffe0'
        t.card.elevation = 8
        t.card.width     = '430px'

        display(t.draw())


    .. figure:: figures/textlist.png
       :scale: 100 %
       :alt: textlist widget

       Textlist widget for displaying textual information.
   """

    # Initialization
    def __init__(self,
                 titles,
                 texts,
                 titlesbold=[],
                 titlefontsize=12,
                 textfontsize=12,
                 titlecolumn=4,
                 textcolumn=8,
                 titlecolor='black',
                 textcolor='black',
                 lineheightfactor=1.5):
        self.titlefontsize    = titlefontsize
        self.textfontsize     = textfontsize
        self.titlecolumn      = titlecolumn
        self.textcolumn       = textcolumn
        self.titlecolor       = titlecolor
        self.textcolor        = textcolor
        self.lineheightfactor = lineheightfactor
        self.div  = v.Divider(class_='pa-0 ma-0 ml-1 mr-1', vertical=False)
        self.card = v.Card(flat=True)
        self.update(titles, texts, titlesbold)
        
        
    # Update the widget with new titles and strings
    def update(self, titles, texts, titlesbold=[]):
        self.card.children = []
        children = []
        lineheight = "line-height: %dpx;"%(int(self.lineheightfactor*(max(self.titlefontsize,self.textfontsize)))) # To ensure vertical center alignment
        for title,text in zip(titles, texts):
            c1 = v.Col(cols=self.titlecolumn,  class_="pa-0 ma-0 ml-3",
                        children=[v.Html(tag='div', class_='pa-0 ma-0', children=[str(title)],
                                        style_='color: %s; font-weight: 450; text-transform: none; font-size: %dpx; %s'%(self.titlecolor, self.titlefontsize,lineheight))],
                        style_="overflow: hidden;")

            if title in titlesbold: weight = 500
            else:                   weight = 350

            c2 = v.Col(cols=self.textcolumn,  class_="pa-0 ma-0",
                       children=[v.Html(tag='div', class_='pa-0 ma-0', 
                                        children=[str(text)], 
                                        style_='color: %s; font-weight: %d; text-transform: none; font-size: %dpx; %s'%(self.textcolor, weight,self.textfontsize,lineheight))],
                       style_="overflow: hidden;")
            children.append(v.Layout(class_="pa-0 ma-0", children=[c1,c2], style_="overflow: visible;"))
            children.append(self.div)

        self.card.children = children
        
   
    # Returns the vuetify object to display
    def draw(self):
        """Returns the ipyvuetify object to display (the internal v.Card widget)"""
        return self.card




"""Custom sample page with left and bottom panels"""
# Author(s): Davide.De-Marchi@ec.europa.eu
# Copyright © European Union 2024
# 
# Licensed under the EUPL, Version 1.2 or as soon they will be approved by 
# the European Commission subsequent versions of the EUPL (the "Licence");
# 
# You may not use this work except in compliance with the Licence.
# 
# You may obtain a copy of the Licence at:
# https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12

# Unless required by applicable law or agreed to in writing, software
# distributed under the Licence is distributed on an "AS IS"
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied.
# 
# See the Licence for the specific language governing permissions and
# limitations under the Licence.

# Imports
import ipyvuetify as v

# Vois imports
from vois.vuetify import settings, iconButton


#####################################################################################################################################################
# Dynamic button to open and close a panel
#####################################################################################################################################################
class dynamicButton():
    
    def __init__(self,
                 icon1='mdi-menu-left',
                 icon2='mdi-menu-right',
                 color1=settings.color_first,
                 color2=settings.color_first,
                 tooltip1='Close panel',
                 tooltip2='Open panel',
                 x1='10vw', y1='10vh',
                 x2='1vw',  y2='10vh',
                 onclick1=None,
                 onclick2=None):
        
        self.icon1    = icon1
        self.icon2    = icon2
        self.color1   = color1
        self.color2   = color2
        self.tooltip1 = tooltip1
        self.tooltip2 = tooltip2
        self.x1       = x1
        self.y1       = y1
        self.x2       = x2
        self.y2       = y2
        self.onclick1 = onclick1
        self.onclick2 = onclick2
        
        self.ib = iconButton.iconButton(icon=self.icon1, onclick=self.onclick, large=True, tooltip=self.tooltip1)
        
        self.nav = v.NavigationDrawer(stateless=True, permanent=True, floating=True, fixed=True, left=True, color="transparent", width=50, height=50,
                                      style_='left: %s; top: %s; z-index:1000;'%(self.x1, self.y1), class_='pa-0 ma-0', children=[self.ib.draw()])
        self.pos1 = True

        
    def draw(self):
        return self.nav
    
    
    def onclick(self):
        self.pos1 = not self.pos1
        self.setpos()
        
        
    def setpos(self):
        if self.pos1:
            self.nav.style_      = 'left: %s; top: %s; z-index:1000;'%(self.x1, self.y1)
            self.ib.tooltip      = self.tooltip1
            self.ib.btn.children = [v.Icon(children=[self.icon1], large=self.ib.large, small=self.ib.small, x_large=self.ib.x_large, x_small=self.ib.x_small)]
            self.ib.btn.color    = self.color1
            if not self.onclick2 is None: self.onclick2()
        else:
            self.nav.style_      = 'left: %s; top: %s; z-index:1000;'%(self.x2, self.y2)
            self.ib.tooltip      = self.tooltip2
            self.ib.btn.children = [v.Icon(children=[self.icon2], large=self.ib.large, small=self.ib.small, x_large=self.ib.x_large, x_small=self.ib.x_small)]
            self.ib.btn.color    = self.color2
            if not self.onclick1 is None: self.onclick1()

    @property
    def color(self):
        return self.color1
        
    @color.setter
    def color(self, c):
        self.color1 = c
        self.color2 = c
        self.ib.btn.color = c